;
; $Header: E:/HAM/EVM/TEST/RCS/APT.ASM 1.6 1997/01/28 13:03:17 dbraun Exp $
;
; $Log: APT.ASM $
; Revision 1.6  1997/01/28 13:03:17  dbraun
; Changed but filter coeffs, and added conditional updating
; of VCO frequency.
;
; Revision 1.5  1997/01/27 19:21:41  dbraun
; Added bit filter.
;
; Revision 1.4  1997/01/27 10:05:31  dbraun
; Added SamplesPerPixel and other details.
;
; Revision 1.3  1997/01/27 00:11:05  dbraun
; Simplified the vector arithmetic a little.
;
; Revision 1.2  1997/01/27 00:05:50  dbraun
; Got phase correction correct.
;
; Revision 1.1  1997/01/26 23:53:16  dbraun
; Initial revision
;
;

	title 'APT WESAT interface for JVFAX by N1OWU'
	page	132,79
	opt	rc
	opt	mu

	nolist
	include 'leonid'
	include 'ioequlc'
	include 'intequlc'
	list


DCDLED  macro mode      ; DCDLED clr/set/chg
	b\mode #13,X:$FFE4
	endm

UpLED   macro mode      ;UP line (a red LED connected)
        b\mode #1,X:$FFE4
        endm

DownLED macro mode      ;DOWN line (a red LED connected)
        b\mode #2,X:$FFE4
        endm


buflen	equ	256

SampleFreq	equ	9600.0		    ; sampling frequency
CarrierFreq	equ	2400.0		    ; carrier freq
SamplesPerPixel equ	2	; each sample is sent out the serial port
				; JFAX is set up for 4800 pixels/sec.

TestFreq	equ	2395.0		    ; test freq

; Should be as narrow as possible for the number of taps
CarrierFilterBW	equ	200.0
CarrierFilterLen equ  64

; The PLL loop gain.  A critical parameter...
LoopGain equ	0.003*(2.0*CarrierFreq/SampleFreq)

; This is a parameter of the IIR LPF for a 2nd order PLL.
; If it is very close to 1.0, the LPF is effectively disabled,
; turning it into a 1st order PLL.
LpfDecayCoeff equ    1.0/1.000001

; The phase correction factor needed to get the VCO
; phase to exactly match the signal phase.  In theory this
; factor could be derived from the filter and VCO properties,
; but I will just determine it empirically.  I would
; expect it to be a multiple of 90 or 45 degrees, though...

PhaseCorrection	equ 0.0

; Signal filter bandwidth should be wide enough for all the signal...
SignalFilterBW	equ	2400.0
SignalFilterLen equ  64

BitFilterLen	equ	3

; RxOutScale Needs to be tuned to adjust contrast 
RxOutScale      equ 1.25   ; OK for METOER sats


NewBaudRate     equ 57600   ;We send a lot of pixels/second thus we need
			    ;this high speed on the serial port

SPY	equ	1
SelfTest equ    0     ; If nonzero, it demodulates an internally generated tone.


; Register Usage:

;r2, n2, m2 are for addressing the sample buffer.
;r6, m6 are for addressing the demodulated bit filter.
;r7, m7 must not be used: SSI interrupts
;r3, m3 must not be used: SCI interrupts and LEONID code

; generally:
; r0, m0, n0 point to data during filtering MACs
; r4 points to filter coeffs during filtering MACs.  (m4,n4 not necessary)



	org	p:user_code


	ori	#$04,omr			    ; enable on-chip sine table

	move		    #buffer+2,r7	    ; codec sample buffer ptr
	move		    #buflen*4-1,m7

	move		    #buffer,r2		    ; sample buffer read ptr
	move		    #4,n2
	move		    #buflen*4-1,m2

	move		    #<BitFilter,r6	    ; bit filter ptr
	move		    #<BitFilterLen-1,m6

; N1OWU:  Activate the relay that switches the analog I/O
; to the 9600 baud FSK port.  It's connected to bit 9 of port B.
; This is correct for the IC-821 setup.  It causes the EVM
; to get the radio's data output, not the audio output. 

      bset	#9,x:$ffe4 


; Set up correct baud rate.
; Note: xtal is defined in leonid.asm

	if !SPY								  
	  bclr #12,X:<<$FFF0    ;disable SCI Tx interrupts		  
	  if NewBaudRate        ;set new baud rate (copied from LEONID)	  
	   bclr #11,X:<<$FFF0    ;disable SCI Rx interrupts		  
	   jclr #0,X:<<$FFF1,*  ;wait until all data is sent out	  
	   movep #(xtal+2*16*NewBaudRate)/(2*2*16*NewBaudRate)-1,X:<<$FFF2
	   bset #11,X:<<$FFF0    ;re-enable only SCI Rx interrupts		  
	  endif								  
	endif								  


; SampleFreq = 9600 kHz, line input, line output, gain to match radio
	ctrlcd	1,r2,buflen,MIC,10.0,0.0,LINEO|HEADP,12.0,12.0
	opencd	SampleFreq/1000.0,HPF

MainLoop

	.loop	#SamplesPerPixel

; wait for one sample
	waitblk r2,buflen,1

	if SelfTest
	  ; In self test mode, we ignore the input and put in its
	  ; place an oscillator output
	  move    #>2*TestFreq/SampleFreq,a
	  move    x:<TestPhase,x0	
	  add	  x0,a		  ;needs to be able to wrap around
	  move    a1,x:<TestPhase
	  move	  a1,x0
	  jsr	  <IQ
	  move    a1,X:(r2)
	endif

; Put the input signal through a wide bandpass Hilbert
; Transform filter to generate I/Q samples of the signal


	move		    #SignalFilterI,r4
	move		    #SignalFilterQ,r5
	move		    #-4,n0
	move		    m2,m0
	move		    r2,r0     ; point to left channel samples
	clr	a
	clr	b
	move		                  y:(r4)+,y0
	move		    x:(r0)+n0,x0  y:(r5)+,y1
	.loop	#SignalFilterLen-1
	mac	x0,y0,a                   y:(r4)+,y0
	mac	x0,y1,b    x:(r0)+n0,x0  y:(r5)+,y1
	.endl
	macr	x0,y0,a
	macr	x0,y1,b
	
	move	ab,l:<SignalIQ

; Mix the (complex) signal with the PLL oscillator (which is locked
; to the signal carrier) to demodulate the signal.  Since we are
; using complex data for all signals, are calculating
; a vector dot product.

	move    x:<VCOphase,a	

; Apply the phase correction factor needed to get the VCO
; phase to exactly match the signal phase.  In theory this
; factor could be derived from the filter and VCO properties,
; but I will just determine it empirically.  I would
; expect it to be a multiple of 90 or 45 degrees, though...

	move	#>PhaseCorrection,x0
	add	x0,a 		; needs to be able to wrap around.
	move	a1,x0

	jsr	<IQ
	move	ab,l:<VCOIQ  ; save for debugging

        move a,y1
        move L:<SignalIQ,x
        mpy x1,y1,a b,y0
        macr x0,y0,a

        ; Stuff the data value into the bit filter
        move	a,X:(r6)+
        ; Save the demodulated data value for debugging
        move	a,X:<RxData

	if SPY
	  jsr <SpyA
        endif

        ; A clean APT signal should always demodulate to a
        ; positive value, since its phase is constant.
	; If we get a negative value, it
        ; must be due to noise corrupting the signal.  If so,
        ; do not attempt to update the PLL, since we will only
        ; make it worse.
        ; BUT: will this make it harder for the PLL to achieve lock?

        tst	a
        .if <gt>

	; Put the left channel through a narrow bandpass Hilbert
	; Transform filter to generate I/Q samples of the carrier.
	move		    #CarrierFilterI,r4
	move		    #CarrierFilterQ,r5
	move		    #-4,n0
	move		    m2,m0
	move		    r2,r0     ; point to left channel samples

	clr	a
	clr	b
	move		                  y:(r4)+,y0
	move		    x:(r0)+n0,x0  y:(r5)+,y1
	.loop	#CarrierFilterLen-1
	mac	x0,y0,a                   y:(r4)+,y0
	mac	x0,y1,b    x:(r0)+n0,x0  y:(r5)+,y1
	.endl
	macr	x0,y0,a
	macr	x0,y1,b


	; Calculate the phase of the input signal, and subtract it
	; from the phase of the VCO.
	; This approach is computationally expensive, but produces
	; an output that is completely linear for a full 360 degrees
	; of phase difference.

	move	ab,l:<CarrierIQ
	jsr	<RaughNormIQ   ; Normalize to avoid loss of accuracy in Phase
	jsr	<Phase
	move	a,x:<CarrierPhase

	; Calculate the phase difference.
	move    x:<VCOphase,x0	
	move	x:<CarrierPhase,a
	sub	x0,a
  	move	a1,x0		; needs to be able to wrap around

	; Do the loop low-pass filter.  Input and output in x0.
	; As the LpfDecayCoeff approaches 1.0, this code is effectively
	; a no-op.  Input in x0.

	clr	a
	move	#LpfDecayCoeff,x1
	mac	x0,x1,a 
	move	x:<LoopFilterTmp1,x0
	move	#1.0-LpfDecayCoeff,x1
	macr	x0,x1,a 
	move	a,x:<LoopFilterTmp1
	move	a,x0

	; Use the output of the LPF (in x0) to control the PLL freq.
    	move	#LoopGain,x1
	mpyr	x0,x1,a     #>2.0*(CarrierFreq/SampleFreq),x0
	add	x0,a		    ; calculate the symbol clock frq

	move    a,x:<VCOfreq  ; save it

	.else
	  ; re-use the previous freq value if a new value was not calculated.
	  move    x:<VCOfreq,a
	.endi

	; Advance the VCO phase based on the new (or old) frequency.
	move    x:<VCOphase,x0	
	add	x0,a		  ;needs to be able to wrap around
	move    a1,x:<VCOphase

	; output the NCO signal in b (for debugging)
	; Also advance r2 to the next sample.
	move    x:<VCOphase,x0	
	jsr	<IQ
	move   (r2)+
	move	a,y:(r2)+
	move	a,y:(r2)+
	move    (r2)+

	.endl


; This code is executed only once per pixel, not once per sample.
; So, at 2 samples/pixel, we are using the bit filter as a low pass
; filter, and decimating.

	move r6,r0
	move m6,m0
	move #BitFilterCoeffs,r4
	move (r0)- 		; point to most recent sample

	clr a  X:(r0)-,x0 Y:(r4)+,y0
	.loop #BitFilterLen-1
	  mac x0,y0,a X:(r0)-,x0 Y:(r4)+,y0
	.endl
	macr x0,y0,a          ;a=filtered data


; Scale the signal, and send it out the serial port.
	  move	a,y0		; get the demodulated data value
	  move #>RxOutScale/32768.0,x0
	  mpyr x0,y0,a   #<0,x0       ;limit the data to 0..$FF range
	  tlt x0,a
	  move #>$FF,x0
	  cmp x0,a
	  tgt x0,a

	  ; DB: If the data value is close to zero (black), turn on
	  ; the "down" LED.  If it is close to 255 (white), turn on
	  ; the "up" LED.  Otherwise, turn the DCD LED on.

          DownLED clr
          UpLED clr
          DCDLED clr
	  move #>$40,x0
	  cmp x0,a
	  .if <le>
	    DownLED set
	  .else
	    move #>$C0,x0
	    cmp x0,a
	    .if <ge>
	      UpLED set
	    .else
	      DCDLED set
	    .endi
	  .endi
	    
	if !SPY
	  jclr #m_tdre,X:<<m_ssr,RxData_end  ;serial Tx empty ?
	  movep a1,X:<<m_stxl    ;output the data to the serial port
        endif   ;Spy
RxData_end


; loop forever
	jmp	<MainLoop



PI      equ     3.14159265358979323846

;this routine computes a cosine/sine pair using the sine ROM
;with a second order (linear+quadrature) approximation between table points
IQ                              ;x0 = angle ( -1 = -PI, +1 = +PI)
        move #>$80,x1   ;shift out 8 most significant bits
        mpy x0,x1,a  #>$FF,x0
        move x0,m0
        and x0,a     #>$100,x0
        or x0,a      #<$40,n0
        move a1,r0      ;put the 8 most significant bits into r0 with offset = $100
        move a0,y0      ;save the remaining bits in y0
        jclr #23,y0,SinTable_lev2
          move (r0)+
SinTable_lev2
        move Y:(r0+n0),x0       ;x0 = coarse cosine
        move Y:(r0),x1          ;x1 = coarse sine
        mpyr x1,y0,a  #PI/256.0,y1
        tfr x0,a  a,x1
        macr -x1,y1,a           ;a = fine cosine
        mpyr x0,y0,b  Y:(r0),x1
        tfr x1,b  b,x1
        macr x1,y1,b  #PI*PI/2.0/65536.0,y1  ;b = fine sine
        mpyr y0,y0,a  a,x0
        move a,y0
        mpyr y0,y1,a
        tfr x0,a  a,y1
        macr -x0,y1,a  b,x1     ;a = super fine cosine
        macr -x1,y1,b           ;b = super fine sine
        rts                     ;x,y are modified
                                ;r0,m0,n0 are modified
                                ;maximum error is about 0.7E-6
                                ;execution time 4+64+4 clock cycles
                                ;including "jsr <IQ_lev2" and "rts"


;This routine scales an I/Q pair such that the larger number
;is within the range +/-0.5..1.0

RaughNormIQ             ;scale an I/Q pair such that the larger number
                        ;is within the range +/-0.5..1.0
        cmpm a,b        ;which number is greater ?
        jge <NormAftB   ;if b then follow the normalization according to it
        tst a           ;otherwise follow a
        jes <RightA     ;if too big (extension in use) the shift to the right
        jnr <DoneA      ;if normalized then we are done
LeftA     asl b         ;otherwise shift left both numbers
	  asl a
        jnn <LeftA      ;until A is normalized
        rts
RightA    asr b         ;if extension set then shift both number to the right
	  asr a
        jes <RightA     ;until a is normalized
DoneA   rts

NormAftB                ;same stuff for b
	tst b
	jes <RightB
        jnr <DoneB
LeftB	  asl a
	  asl b
	jnn <LeftB
        rts
RightB	  asr a
	  asr b
	jes <RightB
DoneB   rts


;The following routine computes the phase of the given I/Q pair
;this routine is not very precise: maximum error is 2E-4 (2E-4/PI radians)
;the error could be made lower but I required that the calculation
;is exact for PI/4, PI/2, PI*3/4, etc.
Phase      ;a = I-part (24 bits), b = Q-part (24 bits)
           ;-1.0 <= a < 1.0, -1.0 <= b < 1.0
        tst b #<0,x0    ;if Q-part negative => turn by PI
        .if <mi>
          neg a #<%10000000,x0
          neg b
        .endi
        tst a           ;if I-part negative => turn by -PI/2
        .if <mi>
          bset #22,x0
          tfr a,b b,a   ;swap I with Q
          neg b         ;negate the new Q
        .endi
        add b,a                 ;add Q to I (what if I+Q >= 1.0 ?)
        asr a                   ;I+Q /= 2 in case it is > 1
        asr b a,x1              ;Q /= 2 because of the above, x1 = I+Q
        andi #$FE,ccr           ;divive Q by (I+Q)
        rep #24
          div x1,b
        tfr x0,a b0,x0          ;x0 = Q div (I+Q), a=approx angle
        tfr x0,b #0.5,x0        ;b = Q/(I+Q)
        sub x0,b #0.25,x0       ;b = Q/(I+Q) - 1/2 =: X
        add x0,a b,x1 #0.63361654,y1 ;angle += 0.25, x1 = X
        mac x1,y1,a x1,x0       ;angle += F1*X, x0 = X
        mpyr x0,x0,b #-0.73728217,y1 ;b = X*X
        mpyr x1,y1,b b,x0       ;b = F3*X, x0 = X*X
        move b,y1
        mac x0,y1,a x0,y0       ;angle += F3*X * X*X, y0 = X*X
        mpyr x0,x0,b #0.85568791,y1 ;b = (X*X)*(X*X)
        mpyr x1,y1,b b,x0       ;b = F5*X, x0 = X*X*X*X
        move b,y1
        mac x0,y1,a             ;angle += F5*X * X*X*X*X
        mpyr x0,y0,b #-0.17769569,y1 ;b = (X*X*X*X)*(X*X)
        mpyr x1,y1,b b,x0       ;b = F5*X, x0 = X*X*X*X*X*X
        move b,y0
        macr x0,y0,a            ;angle += F5*X * X*X*X*X*X*X
        rts             ;a = the phase (-1.0 => -PI, 1.0 => PI)
                        ;b,x,y are modified
                        ;execution time is about 130 cycles (65 instructions)


        if SPY

SpySync move a10,L:<SpySave     ;output: carry=1 => no spy request
        move a2,X:<SpySave+1    ;carry=0 => spy request !
        move x0,Y:<SpySave+1    ;512 words (jsr <SpyA) should follow
        move x1,Y:<SpyCount
        move X:<SpyCount,a
        tst a
        jne <SpyCont
        lookc 0
        jcs <Spy_end
        move #>'S',a
        cmp x0,a
        ori #$01,ccr
        jne <Spy_end
        move #>'P',x0
        putc
        move #>0,x0      ; send a 0 meaning non-complex data
        putc
        move #>512,a
        move a,X:<SpyCount
SpyCont andi #$FE,ccr
        jmp <Spy_end

SpyA    move a10,L:<SpySave
        move a2,X:<SpySave+1
        move x0,Y:<SpySave+1
        move x1,Y:<SpyCount
        move X:<SpyCount,a
        tst a
        jne <Spy_copy

Spy_check
        lookc 0
        jcs <Spy_end
        move #>'S',a
        cmp x0,a
        jne <Spy_end
        move #>'P',x0
        putc
        move #>0,x0      ; send a 0 meaning non-complex data
        putc
        move #>512,a
Spy_copy
        move #>1,x0
        sub x0,a
        move a,X:<SpyCount

        move X:<SpySave,a
        rep #8
          lsr a
        move a1,x0
	putc
        move X:<SpySave,a
        rep #16
          lsr a
        move a1,x0
        putc

Spy_end move L:<SpySave,a10
        move X:<SpySave+1,a2
        move Y:<SpySave+1,x0
        move Y:<SpyCount,x1
        rts

        endif

;*************************************************************************
;Internal data RAM

        LOMEM X:$0000,Y:$0000,L:$0000
        HIMEM X:$00FF,Y:$00FF,L:$00FF

	org	l:user_data

SignalIQ	ds	1
CarrierIQ	ds	1
VCOIQ		ds	1

        if SPY
SpySave dc 0,0
SpyCount dc 0
        endif

LastL = *

	org	x:LastL

LoopFilterTmp1	dc	0
LoopFilterTmp2	dc	0
VCOfreq dc	2*CarrierFreq/SampleFreq
VCOphase dc	0
RxData	dc	0
CarrierPhase	dc	0
TestPhase dc	0

BitFilter dsm BitFilterLen





;*************************************************************************
;External data RAM

        LOMEM X:$0200,Y:$0200,L:$0200
        HIMEM X:$3800,Y:$3800,L:$3800

	org	l:$0200

buffer	dsm	buflen*4

LastL = *

	org	y:LastL


w0      equ     0.35875 ;coeff. for Blackman-Harris 4 term
w1      equ     0.48829 ;minimum sidelobe window (copied from Motorola BBS)
w2      equ     0.14128
w3	equ	0.01168

; Carrier filter parameters
LowFreq		= (CarrierFreq-(CarrierFilterBW/2))/SampleFreq
UppFreq		= (CarrierFreq+(CarrierFilterBW/2))/SampleFreq

CarrierFilterI

time = -@cvf(CarrierFilterLen/2)+0.5
count   set 0
        dup CarrierFilterLen
angle = PI*time/@cvf(CarrierFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = (@sin(2.0*PI*time*UppFreq)-@sin(2.0*PI*time*LowFreq))/(PI*time)
time = time+1.0
        dc  Window*Filter
count   set count+1
        endm


CarrierFilterQ

time = -@cvf(CarrierFilterLen/2)+0.5
count   set 0
        dup CarrierFilterLen
angle = PI*time/@cvf(CarrierFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = (-@cos(2.0*PI*time*UppFreq)+@cos(2.0*PI*time*LowFreq))/(PI*time)
time = time+1.0
        dc  Window*Filter
count   set count+1
        endm


; Signal filter parameters
LowFreq		= (CarrierFreq-(SignalFilterBW/2))/SampleFreq
UppFreq		= (CarrierFreq+(SignalFilterBW/2))/SampleFreq

SignalFilterI

time = -@cvf(SignalFilterLen/2)+0.5
count   set 0
        dup SignalFilterLen
angle = PI*time/@cvf(SignalFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = (@sin(2.0*PI*time*UppFreq)-@sin(2.0*PI*time*LowFreq))/(PI*time)
time = time+1.0
        dc  Window*Filter
count   set count+1
        endm


SignalFilterQ

time = -@cvf(SignalFilterLen/2)+0.5
count   set 0
        dup SignalFilterLen
angle = PI*time/@cvf(SignalFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = (-@cos(2.0*PI*time*UppFreq)+@cos(2.0*PI*time*LowFreq))/(PI*time)
time = time+1.0
        dc  Window*Filter
count   set count+1
        endm


BitFilterCoeffs
	; Possibly some fancy math would generate better values...
	dc	0.25,0.5,0.25
