/*
	level1rx.c -- Level 1 synchronous receive routines

  Poor Man's Packet (PMP)
  Copyright (c) 1991 by Andrew C. Payne    All Rights Reserved.

  Permission to use, copy, modify, and distribute this software and its
  documentation without fee for NON-COMMERCIAL AMATEUR RADIO USE ONLY is hereby
  granted, provided that the above copyright notice appear in all copies.
  The author makes no representations about the suitability of this software
  for any purpose.  It is provided "as is" without express or implied warranty.

	July, 1989
	 Andrew C. Payne
*/

/* ----- Includes ----- */

#include <stdio.h>
#include <alloc.h>
#include <conio.h>
#include <mem.h>
#include <dos.h>
#include "pmp.h"
#include "serial.h"

#define	MAXRXQUEUE	20		/* max items in queue */
#define	MAXL1DSIZE	400		/* max size of level 1 data field */


/* ---- Local Variables ----- */

static struct ax25_level1 *RXQueue[MAXRXQUEUE];	/* Receive queue */
static int	RXQSize;			/* number of items in queue */

/* ----- Subroutines ----- */

/* RXLevel1()
	Called whenever there are characters waiting on the KISS serial port,
	handles packet reception.
	Received packets are put in the receive queue.

	DB: Read chars as long as they keep coming, and store in static
	buffer until a whole KISS frame is read.  Then put it into queue.

	Returns the number of packets in the RXQueue.
*/

int RXLevel1(void)
{
	static byte tmpRXQ[MAXL1DSIZE];
	static byte *tmpRXQptr = tmpRXQ;
	static int escaped = FALSE;

	byte	*p;			/* pointer to RX buffer */
	byte	c;			/* current rec'd character */
	struct ax25_level1	*RXB;	/* current RX buffer */
	int	len;

	ShowTXRX(FALSE,TRUE);		/* update screen status */

	while(ReadSerialReady()) {

		c = ReadSerial();

		if ((int)c == -1) {
			;
		} else if (c == FEND) {
			// This is the beginning and/or end of a packet. */
			escaped = FALSE;

			// If we have gotten enough data, this is the end,
			// so copy the completed packet to the RXQueue,
			// prepare to receive the next KISS packet, and
			// return.

			if (tmpRXQptr > tmpRXQ+10) {
				if(RXQSize >= MAXRXQUEUE) {	/* is the RX Queue full? */
					RXQOverflow++;		/* count overflows */
				} else if (tmpRXQ[0] != 0) {
					// The first, "type" byte is supposed to
					// be zero to signify data on port 0.
					RXFrameErr++;
				} else {

					RXB = RXQueue[RXQSize];	/* next queue entry */
					RXQSize++;

					// Since the modem checked and stripped off
					// the CRC bytes, add two zeros at the end
					// so the downstream routines won't get
					// confused.
					*tmpRXQptr++ = 0;
					*tmpRXQptr++ = 0;

					// Copy to the real queue, skipping first byte
					len = tmpRXQptr-tmpRXQ-1;
					memcpy(RXB->data, tmpRXQ+1, len);
					RXB->len = len;
				}
				tmpRXQptr = tmpRXQ;  // Re-initialize.
				ShowTXRX(FALSE,FALSE);
				return RXQSize;
			}
			else if (tmpRXQptr > tmpRXQ) {
				// Ignore runt packets.  The modem
				// should never send these...
				RXFrameErr++;
				tmpRXQptr = tmpRXQ;  // Re-initialize.
				ShowTXRX(FALSE,FALSE);
				return RXQSize;
			}

			// If we haven't gotten any data yet, then this FEND
			// marks the beginning of a new packet.  But since
			// we re-initialized at the endl of the last one,
			// there is nothing that has to be done.
			// Note: many FENDs in a row is not a problem!

		} else if (c == FESC) {
			escaped = TRUE;
		} else {
			if (escaped) {
				if (c == TFESC)
					c = FESC;
				else if (c == TFEND)
					c = FEND;
				else
					RXFrameErr++;
				// If c is something else, we should ignore it...

				escaped = FALSE;
			}
			
			/* check for tmp RX buffer overflow */
			if(tmpRXQptr >= tmpRXQ+MAXL1DSIZE-2) {	/* packet too long */
				RXBOverflow++;
				tmpRXQptr = tmpRXQ;  // Re-initialize.
				ShowTXRX(FALSE,FALSE);
				return RXQSize;
			}

			// If everything is OK, save the byte.
			*tmpRXQptr++ = c;
		}
	}

	// No more characters available at the moment.
	return RXQSize;
}


/* RXProcess()
	Processes the packets sitting in the level1 RX queue.  Valid
	packets are handed up to the AX.25 routines.
*/
void RXProcess(void)
{
	int	i;

/* empty RX queue? */
	if(RXQSize == 0)
		return;

/* process RXQueue items */
	for(i=0; i<RXQSize; i++) {

/* if CRC is good, show & handle packet */
// KISS packets have no CRCs, so don't check it.
		// if(CRCCheck(RXQueue[i])) {
		if (TRUE) {
			RXCount++;
			AX25Level2(RXQueue[i]);		/* upcall */
		} else {
			RXCRCErr++;
			if(PassMode) {
				uprintf(BrightAttr,"~");
				AX25Level2(RXQueue[i]);	/* upcall */
			}
		}
	}

/* empty RX Queue */
	RXQSize = 0;
}

/* RXInit()
	Initialize the RX routines:  allocate the level 1 receive buffers.
*/
void RXInit()
{
	int	i;

/* allocate the RX Queue */
	for(i=0; i<MAXRXQUEUE; i++) {
		if((RXQueue[i] = malloc(sizeof(struct ax25_level1) + MAXL1DSIZE))
			== NULL)
				OutOfMemory();
	}


	RXQSize = 0;		/* empty queue */
}
