/*
	param.c -- Reads paramter file when program is started

  Poor Man's Packet (PMP)
  Copyright (c) 1991 by Andrew C. Payne    All Rights Reserved.

  Permission to use, copy, modify, and distribute this software and its
  documentation without fee for NON-COMMERCIAL AMATEUR RADIO USE ONLY is hereby
  granted, provided that the above copyright notice appear in all copies.
  The author makes no representations about the suitability of this software
  for any purpose.  It is provided "as is" without express or implied warranty.

	August, 1989
	Andrew C. Payne
*/

/* ---- Includes ----- */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <stdarg.h>
#include "pmp.h"

static int 	domycall(char *p),dotxdelay(char *p),dotxtail(char*p),
		dopersist(char *p),doslottime(char *p),
		dofullduplex(char*p), dopthresh(char *p),
		dofrack(char *p), doretry(char *p), donormal(char *p),
		doinverse(char *p), dobright(char *p), dostatus(char *p),
		dosound(char *p), doctext(char *p), dobtext(char *p),
		doserialport(char *p), docheck(char *p), domaxframe(char *p),
		domessage(char *p), docapture(char *p), dodefine(char *p),
		dobeacon(char *p), dobaddr(char *p),
		doautowrap(char *p), dopaclen(char *p);

extern int SetAX25Addr(struct ax25_addr *p, char *c);

/* ----- Command list Structure ----- */

static struct param_cmd pcmds[] = {
	{ "mycall",	domycall },
	{ "txdelay",	dotxdelay },
	{ "txtail",	dotxtail },
	{ "persistence", dopersist },
	{ "slottime",	doslottime },
	{ "fullduplex",	dofullduplex },
	{ "pthresh",	dopthresh },
	{ "frack",	dofrack },
	{ "retry",	doretry },
	{ "normal",	donormal },
	{ "inverse",	doinverse },
	{ "bright",	dobright },
	{ "status",	dostatus },
	{ "message",	domessage },
	{ "sound",	dosound },
	{ "ctext",	doctext },
	{ "btext", 	dobtext },
	{ "serialport",	doserialport },
	{ "check",	docheck },
	{ "maxframe",	domaxframe },
	{ "capture", 	docapture },
	{ "define",	dodefine },
	{ "beacon",	dobeacon },
	{ "baddr",	dobaddr },
	{ "autowrap",	doautowrap },
	{ "paclen",	dopaclen },
	{ NULL,		NULL } };

static int	line;		/* current line number */

/* error()
	Show error message.
*/
void error(char *format, ...)
{
	char	s[100];
	va_list	argptr;

	cprintf("Error on line %d in configuration file %s :\r\n      ",line,PMP_CONFIG_FILE);
	va_start(argptr,format);
	vsprintf(s, format, argptr);
	va_end(argptr);
	cprintf(s);
}

/* InitParameters()
	Sets initial parameters.
*/
void InitParameters()
{
	int	i;

	SetAX25Addr(&MyCall,"NOCALL");
	Sound = FALSE;
	Pthresh = 0;
	Frack = 3;
	Retry = 7;
	MaxFrame = 7;
	Sound = TRUE;
	*btext = '\0';
	strcpy(baddr, "CQ");
	BeaconInt = 0;
	nctexts = 0;
	CaptureBuffer = 2048;
	AutoWrap = 0;
	Paclen = 128;
	for(i=1; i<=10; i++)
		*fkeys[i] = '\0';

	ClockSkew = FALSE;

	SerialPort = 1;
	SerialSpeed = 9600;

	TXDelay = -1;  /* -1 means let modem use its default value. */
	TXTail = -1;
	Ppersist = -1;
	SlotTime = -1;
	FullDuplex = -1;
}

/* ReadParameters()
	Opens and reads the parameter file.
	The parameter argv0 is argv[0], obviously...

	Returns TRUE if error.
*/
int ReadParameters(char *argv0)
{
	FILE	*infile;
	char	s[100];
	char	cmd[80];
	char	*p;
	int	i;

/* open the file for input */

	if (*ParamFname) {
		// If the user specified a particular file, try to
		// open it exactly as given.
		if((infile = fopen(ParamFname,"r")) == NULL) {
			cprintf("Error -- Can't find configuration file '%s'   ",ParamFname);
			return TRUE;
		}
	}
	else {
		// If the user did not specify a filename, look for the
		// default-named file in the CWD, then according to
		// the .EXE location (based on argv[0]).

		strcpy(ParamFname, PMP_CONFIG_FILE);
		infile = fopen(ParamFname,"r");

		if (!infile) {
			strcpy(ParamFname, argv0);
			// Strip off the .exe file name from argv[0]
			p = ParamFname + strlen(ParamFname) - 1;
			while (p >= ParamFname && *p != '\\' && *p != ':') 
				*p-- = '\0';
			p++;

			// Add on the file name
			strcpy(p, PMP_CONFIG_FILE);

			if((infile = fopen(ParamFname,"r")) == NULL) {
				cprintf("Error -- Can't find configuration file '%s'   ",PMP_CONFIG_FILE);
				return TRUE;
			}
		}
	}

/* read through lines till end of file */
	line = 0;
	while(fgets(s,100,infile) != NULL) {
		line++;
		p = sob(s);
		if(*p && *p != '#') {
			i = dispatch(p, pcmds);
			if(i == FAIL) {
				p = extract(p, cmd);
				error("Unknown command '%s'",cmd);
			}
			if(i) {
				fclose(infile);
				return TRUE;
			}
		}
	}
	fclose(infile);
	return FALSE;
}

/* ----- Command Handlers ----- */

/* domycall(s)
	Sets the callsign of this node.
*/
static int domycall(char *s)
{
	if(SetAX25Addr(&MyCall,s)) {
		error("Invalid callsign");
		return TRUE;
	} else
		return FALSE;
}

/* dotxdelay(s)
	Sets the transmitter delay.  This is the wait after the TX is keyed
	and before data is sent.  The time is given in hundreths of seconds.
	This parameter is simply relayed to the modem.
*/
static int dotxdelay(char *s)
{
	TXDelay = atoi(s);
	if (TXDelay < 0 || TXDelay > 255) {
		TXDelay = -1;
		error("Invalid TXDELAY value");                                      
		return TRUE;    
	} else
		return FALSE;
}

/* dotxtail(s)
	Sets the transmitter tail.  This is how long the transmitter is
	left on after data is sent.  The time is given in hundreths of seconds.
	This parameter is simply relayed to the modem.
*/
static int dotxtail(char *s)
{
	TXTail = atoi(s);
	if (TXTail < 0 || TXTail > 255) {
		TXTail = -1;
		error("Invalid TXTAIL value");                                      
		return TRUE;    
	} else
		return FALSE;
}

/* dopersist(s)
	Sets the P-persistence parameter.  
	The value ranges from 0 to 255.
	This parameter is simply relayed to the modem.
*/
static int dopersist(char *s)
{
	Ppersist = atoi(s);
	if (Ppersist < 0 || Ppersist > 255) {
		Ppersist = -1;
		error("Invalid PERSIST value");                                      
		return TRUE;    
	} else
		return FALSE;
}

/* doslottime(s)
	Sets the slottime parameter.  
	The value ranges from 0 to 255, in hundredths of a second.
	This parameter is simply relayed to the modem.
*/
static int doslottime(char *s)
{
	SlotTime = atoi(s);
	if (SlotTime < 0 || SlotTime > 255) {
		SlotTime = -1;
		error("Invalid SLOTTIME value");                                      
		return TRUE;    
	} else
		return FALSE;
}

/* dofullduplex(s)
	Sets the full-duplex parameter.  
	The value is either 1 or 0.
	This parameter is simply relayed to the modem.
*/
static int dofullduplex(char *s)
{
	FullDuplex = atoi(s);
	if (FullDuplex < 0 || FullDuplex > 1) {
		FullDuplex = -1;
		error("Invalid FULLDUPLEX value");                                      
		return TRUE;    
	} else
		return FALSE;
}


/* dopthresh(s)
	This parameter is a slight modification to AX.25 Version 2.0.  If
	an I-packet has a data field smaller than this parameter, it is
	always resent w/ a poll after a timeout, instead of polling w/ RR.
*/
static int dopthresh(char *s)
{
	Pthresh = atoi(s);
	return FALSE;
}

/* dofrack(s)
	This parameter is used to calculate the timeout time before retrying.
*/
static int dofrack(char *s)
{
	Frack = atoi(s);
	return FALSE;
}

/* docheck(s)
	This parameter is used as a keep-alive timer (in seconds).
*/
static int docheck(char *s)
{
	Check = atoi(s);
	return FALSE;
}

/* doretry(s)
	This parameter specifies the maximum number of retries before a
	connection is dropped.
*/
static int doretry(char *s)
{
	Retry = atoi(s);
	return FALSE;
}

/* domaxframe(s)
	This parameter specifies the maximum number of frames allowed
	outstanding (e.g. unACKed).
*/
static int domaxframe(char *s)
{
	MaxFrame = atoi(s);
	if(MaxFrame < 1 || MaxFrame > 7) {
		error("MAXFRAME must be between 1 and 7");
		return TRUE;
	} else
		return FALSE;
}

/* donormal(s)
	Sets the normal screen attribute.
*/
static int donormal(char *s)
{
	NormalAttr = atoi(s);
	return FALSE;
}

/* doinverse(s)
	Sets the inverse screen attribute.
*/
static int doinverse(char *s)
{
	InvAttr = atoi(s);
	return FALSE;
}

/* dobright(s)
	Sets the bright screen attribute
*/
static int dobright(char *s)
{
	BrightAttr = atoi(s);
	return FALSE;
}

/* dostatus(s)
	Sets the status line attribute
*/
static int dostatus(char *s)
{
	StatusAttr = atoi(s);
	return FALSE;
}

/* domessage(s)
	Sets the message line attribute
*/
static int domessage(char *s)
{
	MsgAttr = atoi(s);
	return FALSE;
}

/* dosound(s)
	Enables/disables sound.
*/
static int dosound(char *s)
{
	Sound = atoi(s);
	return FALSE;
}

/* doctext(s)
	Sets CTEXT lines, the connect welcome string.
*/
static int doctext(char *s)
{
	if(nctexts >= MAXCTEXTS) {	/* check for too many lines */
		error("Too many CTEXT lines.");
		return TRUE;
	}

	ctext[nctexts++] = strdup(s);
	return FALSE;
}

/* dobtext(s)
	Sets btext, the beacon/CQ string.

	btext includes a carriage return.
*/
static int dobtext(char *s)
{
	btext = strdup(s);
	return FALSE;
}

/* doserialport(s)
	Sets the pparameters fo ther serial port to use.
*/
static int doserialport(char *s)
{
	SerialPort = strtoul(s, &s, 0);
	s = sob(s);
	SerialSpeed = strtoul(s, &s, 0);
	s = sob(s);
	return FALSE;
}

/* docapture(s)
	This buffer specifies the size of the capture buffer.  The larger this
	number, the less disk writes required.  This number should be a multiple
	of a sector size (512).
*/
static int docapture(char *s)
{
	CaptureBuffer = atol(s);
	if(CaptureBuffer < 512 || CaptureBuffer > 65536) {
		error("CAPTURE must be between 512 and 65536 bytes");
		return TRUE;
	}
	return FALSE;
}

/* dodefine(s)
	Defines function keys (F1 through F10).
*/
static int dodefine(char *s)
{
	char	keyname[80];
	int	keynumber;

/* extract function key */
	s = extract(s, keyname);
	keynumber = atoi(keyname + 1);
	if(*keyname != 'f' || keynumber < 0 || keynumber > 10) {
		error("Function key F1-F10 expected");
		return TRUE;
	}

/* extract key definition */
	if(convertkey(fkeys[keynumber-1], s)) {
		error("Error in definition for function key %s",keyname);
		return TRUE;
	} else
		return FALSE;
}

/* dobeacon(s)
	Sets the interval for beaconing (in seconds).
*/
static int dobeacon(char *s)
{
	BeaconInt = atoi(s);
	return FALSE;
}

/* dobaddr(s)
	Sets the destination call and path for beacon packets.
*/
static int dobaddr(char *s)
{
	baddr = strdup(s);
	return FALSE;
}

/* doautowrap(s)
	Sets the column at which automatic word wrap is active.
*/
static int doautowrap(char *s)
{
	AutoWrap = atoi(s);
	if(AutoWrap && (AutoWrap < 10 || AutoWrap > 79)) {
		error("AUTOWRAP must be zero or between 10 and 79 bytes");
		return TRUE;
	}
	return FALSE;
}

/* dopaclen(s)
	Sets the maximum transmitted packet length.
*/
static int dopaclen(char *s)
{
	Paclen = atoi(s);
	if(Paclen < 1 || Paclen > 256) {
		error("PACLEN must be between 1 and 256 bytes");
		return TRUE;
	}
	return FALSE;
}

