;
; $Header: E:/HAM/EVM/TEST/RCS/POCSAG.ASM 1.3 1996/06/21 06:17:31 dbraun Exp $
;
; $Log: POCSAG.ASM $
; Revision 1.3  1996/06/21 06:17:31  dbraun
; Changed the way the LEDs are operated.  Now the "up"
; and "down" yellow LEDs are the high- and low-rate DCD indicators,
; and the green DCD LED indicates sync word reception.
;
; Revision 1.2  1996/06/19 19:14:58  dbraun
; Removed "baud" define, since it's in leonid.asm
;
; Revision 1.1  1996/06/19 19:13:00  dbraun
; Initial revision
;
;

; this is an attempt to do some POCSAG decoding - Rob Janssen, PE1CHL
; derived from tst2: lowpass filter left audio channel (cutoff 900Hz),
; fsk: 1200 bits/s FSK modem by Jarkko Vuori
; and g3ruh: 9600 bits/s G3RUH modem by Jarkko Vuori

; register use:
; r0	scratch
; r1	scratch
; r2	codec sample buffer read pointer
; r4	900 Hz low-pass filter
; r5	decision filter 1
; r6	decision filter 2
; r7	codec sample buffer write pointer

	page	132,79
	opt	rc
	title	'POCSAG demodulator'

	nolist
	include 'leonid'
	include 'ioequlc'
	list


fs	equ	9600.0				; sampling freq
fd1	equ	512.0				; decision rate (POCSAG baudrate) #1
fd2	equ	1200.0				; decision rate (POCSAG baudrate) #2

buflen	equ	256

; Symbol synchro
Ksym	equ	0.02				; symbol synchro pll loop gain

; DCD
DCDFil	equ	0.02				; decision error IIR LPF coefficient

sinep	equ	$0100				; ROM sine table starting address
sinel	equ	256				; lenght of sine table

; flags
mkdsn1	equ	1				; time to make a decision 1
mkdsn2	equ	2				; time to make a decision 2
dcd1	equ	3				; DCD 1
dcd2	equ	4				; DCD 2
syncd1	equ	5				; synced to batch 1
syncd2	equ	6				; synced to batch 2

	system	'dsp4'


; macro for N1OWU yellow LED handling
cmdled	macro	mode
	b\mode	#3,x:m_pbd
	endm

; macro for N1OWU green DCD LED handling
dcdled	macro	mode
	b\mode	#13,x:m_pbd
	endm

; macro for yellow "up" LED on N1OWU circuit
upled	macro	mode
	b\mode	#1,x:m_pbd
	endm

; macro for yellow "down" LED on N1OWU circuit
downled	macro	mode
	b\mode	#2,x:m_pbd
	endm


; first order high-pass IIR filter
; input in a, output in a
; frq is -3dB point (fc/fs)
hpass	macro	acc,frq,store
c1	set	-1.0/@cos(6.28319*frq)*(1.0-@sin(6.28319*frq))
	move		    acc,x0
	move		    #>(1.0-c1)/2.0,y0
	mpyr	x0,y0,acc   x:<store,y0
	move		    acc,x:<store
	sub	y0,acc	    x:<store+1,x0
	move		    #>c1,y0
	macr	-x0,y0,acc
	move		    acc,x:<store+1
	endm



	org	p:user_code

	move		    #buffer,r2		; codec sample buffer read ptr
	move		    #4-1-1,n2
	move		    #buflen*4-1,m2

	move		    #buffer+2,r7	; codec sample buffer write ptr
	move		    #buflen*4-1,m7

	ctrlcd	1,r2,buflen,MIC,0.0,0.0,LINEO|HEADP,0.0,0.0
	opencd	fs/1000,NOHPF


	move		    #lotaps,r4		; filter coeff ptr
	move		    #<lolen-1,m4

	move		    #<dfd1,r5		; decision filter sample ptr 1
	move		    #2,n5
	move		    #<dftaps-1,m5

	move		    #<dfd2,r6		; decision filter sample ptr 2
	move		    #2,n6
	move		    #<dftaps-1,m6

; main loop
; update peak values
loop	move		    #0.98,x0
	move		    x:peak,x1
	mpy	x0,x1,a
	move		    a,x:peak
	move		    x:peakf,x1
	mpy	x0,x1,a
	move		    a,x:peakf

;;; set the DCD output according to dcd1,dcd2,synced(cmdled)
;;	btst	#dcd1,x:<flag
;;	jcs	dcdset
;;	btst	#dcd2,x:<flag
;;	jcs	dcdset
;;	cmdled	tst
;;	jcs	dcdset
;;	dcdled	clr				; DCD OFF
;;	jmp	<dcdclr
;;dcdset	dcdled	set				; DCD ON
;;dcdclr

; wait for one sample
	waitblk r2,buflen,1

	move		    r2,r0		; get sample ptr
	move		    #buflen*4-1,m0
	move		    #-4,n0

; get peak value (A/D sample)
	move		    x:(r0),a
	abs	a	    x:peak,x0
	cmp	x0,a
	jlt	<undrpk
	move		    a,x:peak
undrpk

; low-pass filter the left channel
	clr	a
	move		    x:(r0)+n0,x0  y:(r4)+,y0

	rep	#lolen-1
	mac	x0,y0,a	    x:(r0)+n0,x0  y:(r4)+,y0

	macr	x0,y0,a	    (r2)+

; high-pass filter to reject CS4215 offset
	hpass	a,2.0/fs,hpf			; fc = 2 Hz

	move		    a,y:(r2)+		; store filtered value (left)
	move		    a,y:(r2)+n2		; output on right channel as well

; correct sign and store the result
	move	a,y0	    x:sign,x0
	mpy	x0,y0,a
	move	a,y:-(r5)
	move	a,y:-(r6)

; get peak value (filtered)
	abs	a	    x:peakf,x0
	cmp	x0,a
	jlt	<undrpk2
	move		    a,x:peakf
undrpk2

; Symbol synchro 1
ss0	move		    y:(r5),a		    ; comb filter to remove harmonics
	move		    y:(r5+n5),y0
	add	y0,a
	asr	a
	abs	a				    ; get absolute value to generate line at symbol frequence
	move	a,y0

	asr	a	    x:<clocka1,b	    ; remove DC with IIR LPF
	asr	b	    #sinep,r0		    ; and setup sine table pointers
	add	a,b	    y0,a
	move		    b,x:<clocka1
	sub	b,a	    #sinep+1,r1
	move		    a,x:<clocke1

	move		    x:<frqptr1,n0	    ; setup nco
	clr	b	    n0,n1
	move		    #sinel-1,m0
	move		    m0,m1
	move		    y:<frqptr1,b1	    ; convert frqinc fraction to signed
	lsr	b	    y:(r0+n0),x0
	move	b,x1	    y:(r1+n1),b

	sub	x0,b	    l:<frqptr1,y	    ; calculate difference of adjacent sine table entries
	move	b,x0	    y:(r0+n0),b
	macr	x0,x1,b	    l:<frqinc1,a	    ; interpolate output sample

	add	y,a	    #>$100,x0		    ; increment table pointer
	cmp	x0,a	    #>$ff,x0		    ; if over 2PI then make a decision
	jlo	<ss1
	bset	#mkdsn1,x:<flag
ss1	and	x0,a				    ; output generated sample
	move		    a,l:<frqptr1

	move		    x:<clocke1,y0	    ; multiply local clock with derived signal
	move	b,y1
	mpyr	y0,y1,b	    #-Ksym,y0
	move	b,y1
	mpyr	y0,y1,b	    #fd1/fs,y0

	add	y0,b	    #>sinel,y0		    ; and calculate the symbol clock frq
	move	b,y1
	mpy	y0,y1,b
	move		    b10,l:<frqinc1

; Decision filter
	bclr	#mkdsn1,x:<flag			    ; check if time to make a decision
	jcc	<sym2

	move		    #<dfc,r0		    ; yes, first filter the signal from DPLL
	nop
	clr	a	    x:(r0)+,x0	y:(r5)+,y0
	rep	#dftaps-1
	mac	x0,y0,a	    x:(r0)+,x0	y:(r5)+,y0
	macr	x0,y0,a

; calculate eye opening
	move		    #0.65,x0
	move		    x:peakf,x1
	mpy	x0,x1,b
	move	b,x0
	move	a,b
	abs	b
	sub	x0,b        #DCDFil,x1
	move	b,x0
	mpy	x0,x1,b     #(1.0-DCDFil),x0
	move		    x:<eyeflt1,x1
	macr	x0,x1,b
	move		    b,x:<eyeflt1
	jlt	car1
	bset	#dcd1,x:<flag
	downled set
	jmp	<car1e
car1	bclr	#dcd1,x:<flag
	downled clr
car1e

; Get sample and convert to 0/1 in A10
	tst	a
	jmi	<zero1
	clr	a
	move	#>1,a0
	jmp	<one1
zero1	clr	a
one1

; add the new bit into the 32-bit shifter (accu1)
	move	l:<accu1,b10			; get the shifter in b
	addl	a,b				; b:=a+2*b
	move	#>$ff,x0
	and	x0,b				; mask lower byte of upper word
	move	b10,l:<accu1

; check for sync word
	jsr	<chksyn
	jne	<nosync

	jsr	<putb				; got sync! write it.
	move	#>$3a,x0			; ":"
	putc
	move	#>$20,x0
	putc
	bset	#syncd1,x:<flag			; and set flag
	cmdled	set
	move	#>32,x0
	move	x0,x:<bitcnt1
	move	#>16,x0
	move	x0,x:<wrdcnt1
	jmp	<sym2

nosync
	btst	#syncd1,x:<flag
	jcc	<sym2

	move	x:<bitcnt1,a			; check if complete word
	move	#>1,x0
	sub	x0,a
	jne	<noword

	jsr	<putb				; write it out

	move	x:<wrdcnt1,a
	move	#>1,x0
	sub	x0,a
	move	a1,x:<wrdcnt1
	jne	<noend
	bclr	#syncd1,x:<flag			; end of batch, no sync
	cmdled	clr
	move	#>$0d,x0			; send cr/lf
	putc
	move	#>$0a,x0
	putc
	jmp	<endwrd

noend	move	#>$20,x0
	putc
endwrd	move	#>32,a
noword	move	a1,x:<bitcnt1


; Symbol synchro 2
sym2	move		    y:(r6),a		    ; comb filter to remove harmonics
	move		    y:(r6+n6),y0
	add	y0,a
	asr	a
	abs	a				    ; get absolute value to generate line at symbol frequence
	move	a,y0

	asr	a	    x:<clocka2,b	    ; remove DC with IIR LPF
	asr	b	    #sinep,r0		    ; and setup sine table pointers
	add	a,b	    y0,a
	move		    b,x:<clocka2
	sub	b,a	    #sinep+1,r1
	move		    a,x:<clocke2

	move		    x:<frqptr2,n0	    ; setup nco
	clr	b	    n0,n1
	move		    #sinel-1,m0
	move		    m0,m1
	move		    y:<frqptr2,b1	    ; convert frqinc fraction to signed
	lsr	b	    y:(r0+n0),x0
	move	b,x1	    y:(r1+n1),b

	sub	x0,b	    l:<frqptr2,y	    ; calculate difference of adjacent sine table entries
	move	b,x0	    y:(r0+n0),b
	macr	x0,x1,b	    l:<frqinc2,a	    ; interpolate output sample

	add	y,a	    #>$100,x0		    ; increment table pointer
	cmp	x0,a	    #>$ff,x0		    ; if over 2PI then make a decision
	jlo	<ss2
	bset	#mkdsn2,x:<flag
ss2	and	x0,a				    ; output generated sample
	move		    a,l:<frqptr2

	move		    x:<clocke2,y0	    ; multiply local clock with derived signal
	move	b,y1
	mpyr	y0,y1,b	    #-Ksym,y0
	move	b,y1
	mpyr	y0,y1,b	    #fd2/fs,y0

	add	y0,b	    #>sinel,y0		    ; and calculate the symbol clock frq
	move	b,y1
	mpy	y0,y1,b
	move		    b10,l:<frqinc2

; Decision filter
	bclr	#mkdsn2,x:<flag			    ; check if time to make a decision
	jcc	<loop

	move		    #<dfc,r0		    ; yes, first filter the signal from DPLL
	nop
	clr	a	    x:(r0)+,x0	y:(r6)+,y0
	rep	#dftaps-1
	mac	x0,y0,a	    x:(r0)+,x0	y:(r6)+,y0
	macr	x0,y0,a

; calculate eye opening
	move		    #0.65,x0
	move		    x:peakf,x1
	mpy	x0,x1,b
	move	b,x0
	move	a,b
	abs	b
	sub	x0,b        #DCDFil,x1
	move	b,x0
	mpy	x0,x1,b     #(1.0-DCDFil),x0
	move		    x:<eyeflt2,x1
	macr	x0,x1,b
	move		    b,x:<eyeflt2
	jlt	car2
	bset	#dcd2,x:<flag
	upled	set
	jmp	<car2e
car2	bclr	#dcd2,x:<flag
	upled	clr
car2e

; Get sample and convert to 0/1 in A10
	tst	a
	jmi	<zero2
	clr	a
	move	#>1,a0
	jmp	<one2
zero2	clr	a
one2

; add the new bit into the 32-bit shifter (accu2)
	move	l:<accu2,b10			; get the shifter in b
	addl	a,b				; b:=a+2*b
	move	#>$ff,x0
	and	x0,b				; mask lower byte of upper word
	move	b10,l:<accu2

; check for sync word
	jsr	<chksyn
	jne	<nosync2

	jsr	<putb				; got sync! write it.
	move	#>$3b,x0			; ";"
	putc
	move	#>$20,x0
	putc
	bset	#syncd2,x:<flag			; and set flag
	dcdled	set
	move	#>32,x0
	move	x0,x:<bitcnt2
	move	#>16,x0
	move	x0,x:<wrdcnt2
	jmp	<loop

nosync2
	btst	#syncd2,x:<flag
	jcc	<loop

	move	x:<bitcnt2,a			; check if complete word
	move	#>1,x0
	sub	x0,a
	jne	<noword2

	jsr	<putb				; write it out

	move	x:<wrdcnt2,a
	move	#>1,x0
	sub	x0,a
	move	a1,x:<wrdcnt2
	jne	<noend2
	bclr	#syncd2,x:<flag			; end of batch, no sync
	dcdled	clr
	move	#>$0d,x0			; send cr/lf
	putc
	move	#>$0a,x0
	putc
	jmp	<endwrd2

noend2	move	#>$20,x0
	putc
endwrd2 move	#>32,a
noword2 move	a1,x:<bitcnt2

; do it all again
	jmp	<loop


; utility functions

; check for syncword (in b10), adjust sign when inverted syncword found.
; return z status when syncword found
chksyn	move	b1,a
	move	#>$7c,x0			; check for sync word
	cmp	x0,a
	jne	<revsync
	move	b0,a
	move	#>$d215d8,x0
	cmp	x0,a
	rts					; z status when sync

revsync move	#>$83,x0			; check inverted sync
	cmp	x0,a
	jne	<notsyn
	move	b0,a
	move	#>$2dea27,x0
	cmp	x0,a
	jne	<notsyn

	move	x:<sign,a			; swap 0/1
	neg	a
	move	a,x:<sign
	clr	a				; set z status
notsyn	rts

; put b to serial port (32-bit hex)
putb	move	b1,x0
	jsr	<putbyt
	move	b0,a
	rep	#16
	lsr	a
	move	a1,x0
	jsr	<putbyt
	move	b0,a
	rep	#8
	lsr	a
	move	a1,x0
	jsr	<putbyt
	move	b0,x0

; put a byte to serial port (hex)
putbyt	move	x0,x:<x0save
	move	x0,a
	rep	#4
	asr	a
	move	a,x0
	jsr	<putnyb
	move	x:<x0save,x0
putnyb	move	#>$0f,a
	and	x0,a
	move	#>10,x0
	cmp	x0,a
	jge	letter
	move	#>$30,x0
	jmp	<putchr
letter	move	#>$41-10,x0
putchr	add	x0,a
	move	a,x0
	putc
	rts

	org	l:user_data

frqinc1 dc	256*fd1/fs*@pow(2,-23)		; symbol clock synthesizer 1
frqptr1 dc	0

frqinc2 dc	256*fd2/fs*@pow(2,-23)		; symbol clock synthesizer 2
frqptr2 dc	0

accu1	dc	0				; accumulator for data 512 bps
accu2	dc	0				; accumulator for data 1200 bps

buffer	dsm	buflen*4			; sampling buffer

	org	x:user_data+8

sign	dc	-0.9999				; data sign
bitcnt1 dc	32				; bit counter 1
wrdcnt1 dc	16				; word counter 1
bitcnt2 dc	32				; bit counter 2
wrdcnt2 dc	16				; word counter 2
peak	dc	0				; peak input signal
peakf	dc	0				; peak filtered signal
eyeflt1 dc	0				; eye detect filters
eyeflt2 dc	0
flag	dc	0				; some flags

clocka1 dc	0
clocka2 dc	0
clocke1 ds	1
clocke2 ds	1

hpf	ds	2				; data storage for the input DC-cancellation IIR filter
x0save	ds	1

; Decision filter
dfc	dc	-0.00041567696103898691,-0.00081890562387841972,-0.00077287490021642181,1.2127626692456339E-18,0.0012649095635366239
	dc	0.0022206573645156612,0.0019203394725415372,-2.4103681676650389E-18,-0.0027727292080740626,-0.0046459993765459286
	dc	-0.0038631268777868074,3.908467500183978E-18,0.0052444495308750288,0.008578347763639995,0.0069893345319871946
	dc	-5.5625867930785253E-18,-0.009204649706698589,-0.014902880347551255,-0.012059755791003872,7.1756438651449731E-18
	dc	0.015845903566402978,0.02579978381752315,0.02111502337770264,-8.5315994303790973E-18,-0.029056447470843367
	dc	-0.049268154359086921,-0.042777427656427156,9.4358452693819397E-18,0.073704602333730412,0.15797540308255884
	dc	0.22479821342437376,0.25020332089952807,0.22479821342437376,0.15797540308255884,0.073704602333730412
	dc	9.4358452693819397E-18,-0.042777427656427156,-0.049268154359086921,-0.029056447470843367,-8.5315994303790973E-18
	dc	0.02111502337770264,0.02579978381752315,0.015845903566402978,7.1756438651449731E-18,-0.012059755791003872
	dc	-0.014902880347551255,-0.009204649706698589,-5.5625867930785253E-18,0.0069893345319871946,0.008578347763639995
	dc	0.0052444495308750288,3.908467500183978E-18,-0.0038631268777868074,-0.0046459993765459286,-0.0027727292080740626
	dc	-2.4103681676650389E-18,0.0019203394725415372,0.0022206573645156612,0.0012649095635366239,1.2127626692456339E-18
	dc	-0.00077287490021642181,-0.00081890562387841972,-0.00041567696103898691
dftaps	equ	*-dfc



	org	y:user_data+8

dfd1	dsm	dftaps
dfd2	dsm	dftaps

; 900 Hz lowpass filter generated using Parks-McClellan algorithm
;   sampling rate   8000 Hz
;   passband edge    840 Hz
;   stop band edge   960 Hz
;   filter lenght     79 taps
	dsm	256
lolen	equ	79
lotaps	dc	 1.619394e-02,5.805688e-03,3.155693e-03,-1.500173e-03,-6.545769e-03
	dc	-9.963868e-03,-1.022255e-02,-7.061145e-03,-1.754771e-03,3.332559e-03
	dc	 5.765334e-03,4.199639e-03,-8.617942e-04,-7.138900e-03,-1.151400e-02
	dc	-1.147496e-02,-6.426490e-03,1.780802e-03,9.467491e-03,1.274443e-02
	dc	 9.432186e-03,3.507424e-04,-1.070496e-02,-1.840716e-02,-1.827196e-02
	dc	-9.051937e-03,6.112294e-03,2.066623e-02,2.704929e-02,2.031219e-02
	dc	 9.089151e-04,-2.437167e-02,-4.415325e-02,-4.639930e-02,-2.320264e-02
	dc	 2.545267e-02,9.059131e-02,1.564397e-01,2.052803e-01,2.232958e-01	; center tap
	dc	 2.052803e-01,1.564397e-01,9.059131e-02,2.545267e-02,-2.320264e-02
	dc	-4.639930e-02,-4.415325e-02,-2.437167e-02,9.089151e-04,2.031219e-02
	dc	 2.704929e-02,2.066623e-02,6.112294e-03,-9.051937e-03,-1.827196e-02
	dc	-1.840716e-02,-1.070496e-02,3.507424e-04,9.432186e-03,1.274443e-02
	dc	 9.467491e-03,1.780802e-03,-6.426490e-03,-1.147496e-02,-1.151400e-02
	dc	-7.138900e-03,-8.617942e-04,4.199639e-03,5.765334e-03,3.332559e-03
	dc	-1.754771e-03,-7.061145e-03,-1.022255e-02,-9.963868e-03,-6.545769e-03
	dc	-1.500173e-03,3.155693e-03,5.805688e-03,1.619394e-02


	end
