;
;   $Header: E:/HAM/EVM/TEST/RCS/RTCOEFF.ASM 1.5 1997/02/07 15:15:03 dbraun Exp $
;
;   $Log: RTCOEFF.ASM $
;   Revision 1.5  1997/02/07 15:15:03  dbraun
;   Added 'X' and 'x' commands to switch IC-821 input relay.
;
;   Revision 1.4  1997/01/28 14:54:33  dbraun
;   cleaned up control structure.
;
;   Revision 1.3  1996/12/24 05:54:37  dbraun
;   Cleaned up and adjusted levels.
;
;   Revision 1.2  1996/10/25 12:59:10  dbraun
;   Added level checking and LED flashing for overload
;
;   Revision 1.1  1996/10/25 11:40:21  dbraun
;   Initial revision
;
;

;IQ routine from: BPSK1200.ASM, Last edition: 01-JAN-1996
;(c) 1995,1996 Pawel Jalocha, SP9VRC
;
;This software is not to be used for purposes other than amateur radio
;without a written permision of the author.
;Usage for profit or in commercial/military products is explicitly prohibited.

        page 132

        opt mu
        opt mex,cex
        nolist
	include 'leonid'
	list

        scsjmp short    ;force short jumps for .if/.endi, .while/.endw, etc.

        title 'FIR Audio filter with (someday) runtime coefficient calculation'

;*************************************************************************


YellowLED macro mode    ;(a yellow LED connected)
        b\mode #3,X:$FFE4
        endm

UpLED   macro mode      ;UP line (a red LED connected)
        b\mode #1,X:$FFE4
        endm

DownLED macro mode      ;DOWN line (a red LED connected)
        b\mode #2,X:$FFE4
        endm

;*************************************************************************
;compile-time constants


SampleFreq equ 16000.0

CodecBufLen equ 16  ; Since we copy the data to a seperate buffer for
	       ; doing the filtering, this buffer need not be
	       ; the same size as the filter length.

; This must be at least as large as the largest filter
InputBufLen     equ	128

BatchLen        equ     1

LInputGain equ 9.0
RInputGain equ 0.0
OutputAtten equ 3.0

; flags
BOTHIN	 equ	 0 	; Both channels mixed
LEFTIN	 equ	 1	; Left channel input
RIGHTIN	 equ	 2	; Right channel input


CWCenterFreq    equ 800.0
CWBW            equ 250.0  ;Hz bandwidth of CW filter
CWLowFreq = (CWCenterFreq-(CWBW/2))/SampleFreq
CWUppFreq = (CWCenterFreq+(CWBW/2))/SampleFreq


SSBCenterFreq   equ 1500.0
SSBBW           equ 2200.0  ;Hz bandwidth of SSB filter
SSBLowFreq = (SSBCenterFreq-(SSBBW/2))/SampleFreq
SSBUppFreq = (SSBCenterFreq+(SSBBW/2))/SampleFreq

AMLowFreq = 20.0/SampleFreq
AMUppFreq = 4500.0/SampleFreq

SSBFilterLen     equ 64  ;order of filter
CWFilterLen     equ 128
AMFilterLen     equ 64
ThruFilterLen     equ 2  ; smallest possible length


;*************************************************************************
;The actual code (internal/external program RAM)

        LOMEM P:$0000
        HIMEM P:$1FFF

	org	p:user_code

	jsr	Initialize


; main loop, use CODEC as sequencer:
loop    ; get input sample from CODEC
        waitblk r2,CodecBufLen,BatchLen     ;do not use r2, r7,m7 r3,m3

        jsr	getSample
        move    a,y:-(r5)              ;place the sample in input buffer
        jsr	checkLevel

        ; set up the correct filter coefficients and length
	move	X:<CurFilterCoeffs,r4
	move	r5,r0	; point the the right spot in the input buffer
	move	m5,m0
	nop


        ;R0 points to the data, and R4 to the coefficients.
        clr     a           x:(r4)+,x0  y:(r0)+,y0
        .loop    X:<CurFilterLen
           mac  x0,y0,a     x:(r4)+,x0  y:(r0)+,y0   ;Classic FIR filter loop
        .endl
        macr    x0,y0,a

        ;put the filtered data sample into the output buffer.
        move			(r2)+		; skip left channel out
        move                a,y:(r2)+                ; right channel out
        move                    (r2)+                ;point to next input

	jsr     <chk_cmd
        jmp     <loop
        ; end of main loop


getSample
        jset    #LEFTIN,x:<chanflags,_getLeft  
        jset    #RIGHTIN,x:<chanflags,_getRight 

        ;default: get both channels and mix.
	move                x:(r2)+,a  ;store left input sample in a-reg
	move		    x:(r2),x0  ;add right input sample
	add		    x0,a
	;;asr		    a	       ;divide by 2 to normalize
	rts

_getLeft
	move                x:(r2)+,a  ;store left input sample in a-reg
        			       ;and inc ptr to right channel input
	rts

_getRight
        move                (r2)+     ;inc ptr to right channel input
	move                x:(r2),a  ;store right input sample in a-reg
	rts


checkLevel		;if the input sample in A is too high, flash a LED
			;The limit of +/- 0.75 is somewhat arbitrary...
	move	#0.75,x0
	cmp	x0,a
	jgt	_tooHigh
	move	#-0.75,x0
	cmp	x0,a
	jlt	_tooHigh

	; Manage the counter that tracks the flashing LED
	move	#-1,m0
	move	x:levelCounter,r0
	move	r0,a
	tst	a
	jeq	_checkLevelDone   ;Counter is already zero, so exit

	move	(r0)-		 ;If not, decrement and store
	move	r0,x:levelCounter
	move	r0,a		;See if it has finally reached zero
	tst	a
	jeq	_levelLEDOff   ;Counter has reached zero
	rts

_levelLEDOff
	YellowLED clr     ;Turn off LED
_checkLevelDone
	rts

_tooHigh
	YellowLED set
	;set the counter that tracks how long the LED is on
	;At 16k samples/sec, this gives a 0.1 sec flash
	move	#>1600,x0
	move	x0,x:levelCounter
	rts



; Check for control from PC:
chk_cmd lookc   0            ; check if keypressed
        jcs     <_cmdend     ; no key pressed

        move    #>'w',a
	cmp	x0,a
        jeq     <_setW

	move    #>'n',a
	cmp	x0,a
        jeq     <_setN

	move    #>'s',a
	cmp	x0,a
        jeq     <_setS

	move    #>'a',a
	cmp	x0,a
        jeq     <_setA

	move    #>'+',a
	cmp	x0,a
        jeq     <_setP

	move    #>'=',a		;unshifted '+'
	cmp	x0,a
        jeq     <_setP

	move    #>'-',a
	cmp	x0,a
        jeq     <_setM

	move    #>'_',a		;shifted '-'
	cmp	x0,a
        jeq     <_setM

	move    #>'l',a
	cmp	x0,a
        jeq     <_setLeft

	move    #>'r',a
	cmp	x0,a
        jeq     <_setRight

	move    #>'b',a
	cmp	x0,a
        jeq     <_setBoth

	move    #>'X',a	  ; Activate left radio input relay
	cmp	x0,a
        jeq     <_relayOn

	move    #>'x',a	  ; De-activate left radio input relay
	cmp	x0,a
        jeq     <_relayOff

        rts			;ignore unknown commands


_setW
        move    #>'W',x0    ; yes, confirm with W
        putc
        move    #ThruFilter,r0
        move	r0,x:<CurFilterCoeffs
        move    #ThruFilterLen-1,r0
        move	r0,x:<CurFilterLen

        jmp     _cmdend

_setN
        move    #>'N',x0    ; yes, confirm with N
        putc
        move    #CWFilter,r0
        move	r0,x:<CurFilterCoeffs
        move    #CWFilterLen-1,r0
        move	r0,x:<CurFilterLen
        jmp     _cmdend

_setS
        move    #>'S',x0    ; yes, confirm with S
        putc
        move    #SSBFilter,r0
        move	r0,x:<CurFilterCoeffs
        move    #SSBFilterLen-1,r0
        move	r0,x:<CurFilterLen
        jmp     _cmdend

_setA
        move    #>'A',x0    ; yes, confirm with A
        putc
        move    #AMFilter,r0
        move	r0,x:<CurFilterCoeffs
        move    #AMFilterLen-1,r0
        move	r0,x:<CurFilterLen
        jmp     _cmdend

_setP
        ;dec audio attenuation
        jsr	OutputAttenDown
        jcs	_cmdfail    ; Already at maximum.
        move    #>'+',x0    ; OK, confirm with +
        putc
        jmp     _cmdend

_setM
        ;inc audio attenuation
        jsr	OutputAttenUp
        jcs	_cmdfail    ; Already at minimum.
        move    #>'-',x0    ; OK, confirm with -
        putc
        jmp     _cmdend

_setLeft
        move    #>'L',x0
        putc
        move	#>(1<<LEFTIN),x0		;set left input on
        move	x0,x:<chanflags
        UpLED	clr
        DownLED	set
        jmp     _cmdend

_setRight
        move    #>'R',x0
        putc
        move	#>(1<<RIGHTIN),x0	;set right input on
        move	x0,x:<chanflags
        UpLED	set
        DownLED	clr
        jmp     _cmdend

_setBoth
        move    #>'B',x0
        putc
        move	#>(1<<BOTHIN),x0 	;set L+R mix input on
        move	x0,x:<chanflags
        UpLED	set
        DownLED	set
        jmp     _cmdend

_relayOn
        move    #>'X',x0
        putc
	bset	#9,x:$ffe4 
        jmp     _cmdend

_relayOff
        move    #>'x',x0
        putc
	bclr	#9,x:$ffe4 
        jmp     _cmdend

_cmdfail
_cmdend rts


OutputAttenUp                   ;increase the CODEC's output atten (both channels)

        move r2,a0		;decrement r2 to point to output control word
        dec a
        move a0,r2
        clr a #>$3F3F00,x0      ;r2/m2 should point to the CODEC's buffer
        move Y:(r2),a1          ;get the CODEC's output control word
        and x0,a                ;extract the atten bits
	cmp x0,a  #>$010100,x0  ;already maximum ?
        jeq <OutputAttenChange_abort ;if so then abort this attempt
	add x0,a  #>$C0C000,x0  ;if not, increment the atten by 1
	move a1,x1
	move Y:(r2),a1          ;and reload all the control words
	and x0,a  n2,x0         ;in the output buffer
	or x1,a  #<4,n2         ;make n2=4 for a moment
	.loop #CodecBufLen
	  move a1,Y:(r2)+n2
	.endl
	move x0,n2              ;restore n2
        andi #$FE,ccr           ;clear carry
        move r2,a0              ;restore r2
        inc a
        move a0,r2
        rts                     ;modifies a,x0,x1

OutputAttenDown                 ;decrease the CODEC's output atten (both channels)
        move r2,a0		;decrement r2 to point to output control word
        dec a
        move a0,r2
        clr a #>$3F3F00,x0      ;r2/m2 should point to the CODEC's buffer
        move Y:(r2),a1          ;get the CODEC's output control word
        and x0,a  #>$010100,x0  ;extract the atten bits
        sub x0,a  #>$C0C000,x0  ;attempt to decrease the atten
        jcs <OutputAttenChange_abort ;jump if overflow
        move a1,x1
        move Y:(r2),a1          ;reload all the output control words
        and x0,a  n2,x0         ;in the buffer with the new output atten
        or x1,a  #<4,n2         ;n2=4 for a moment
        .loop #CodecBufLen
          move a1,Y:(r2)+n2
        .endl
        move x0,n2              ;restore n2
        andi #$FE,ccr           ;clear carry
        move r2,a0              ;restore r2
        inc a
        move a0,r2
        rts                     ;modifies a,x0,x1

OutputAttenChange_abort
        move r2,a0              ;restore r2
        inc a
        move a0,r2
        ori #$01,ccr            ;set carry to indicate that we couldn't
        rts                     ;change the atten





PI      equ     3.14159265358979323846
EX      equ     2.718281745911
;*********************************************************************


;this routine computes a cosine/sine pair using the sine ROM
;with a second order (linear+quadrature) approximation between table points
        ori #%00000100,omr      ;enable the sine ROM table

IQ                              ;x0 = angle ( -1 = -PI, +1 = +PI)
        move #>$80,x1   ;shift out 8 most significant bits
        mpy x0,x1,a  #>$FF,x0
        move x0,m0
        and x0,a     #>$100,x0
        or x0,a      #<$40,n0
        move a1,r0      ;put the 8 most significant bits into r0 with offset = $100
        move a0,y0      ;save the remaining bits in y0
        jclr #23,y0,SinTable_lev2
          move (r0)+
SinTable_lev2
        move Y:(r0+n0),x0       ;x0 = coarse cosine
        move Y:(r0),x1          ;x1 = coarse sine
        mpyr x1,y0,a  #PI/256.0,y1
        tfr x0,a  a,x1
        macr -x1,y1,a           ;a = fine cosine
        mpyr x0,y0,b  Y:(r0),x1
        tfr x1,b  b,x1
        macr x1,y1,b  #PI*PI/2.0/65536.0,y1  ;b = fine sine
        mpyr y0,y0,a  a,x0
        move a,y0
        mpyr y0,y1,a
        tfr x0,a  a,y1
        macr -x0,y1,a  b,x1     ;a = super fine cosine
        macr -x1,y1,b           ;b = super fine sine
        debug	;TEMP
        rts                     ;x,y are modified
                                ;r0,m0,n0 are modified
                                ;maximum error is about 0.7E-6
                                ;execution time 4+64+4 clock cycles
                                ;including "jsr <IQ_lev2" and "rts"


;-----------------------------------------------------------------------------

; This routine calculates 1/n.
; On entry, X0 is n (a 24-bit signed integer), and at exit, A0 has the
; signed 24-bit fractional reciprocal. A1 and A2 are effectively garbage.
; X0 is not changed.

Recip
	and	#$fe,ccr  ;clear carry bit
	clr	a
	move	#$1,a1    ;put dividend (1) in A
	rep	#24
	div	x0,a
	jclr	#23,x0,pos_recip  ; If n was positive, no need to complement result
	neg	a
pos_recip
	rts

;-----------------------------------------------------------------------------

Initialize      ;initialize registers, buffers, windows, etc.

        ori #%00000100,omr      ;enable the sine ROM table
                                ;this is for the IQ routine

        ;default filter is 5KHZ (for listening to the BBC)
        move    #AMFilter,r0
        move	r0,x:<CurFilterCoeffs
        move    #AMFilterLen-1,r0
        move	r0,x:<CurFilterLen

        ;default input source is Right channel only.
        ;(N1OWU's HF rig)
        move    #>(1<<RIGHTIN),x0
        move	x0,x:<chanflags

        UpLED	set		; LEDs match default input source
        DownLED	clr

        move #InputBuf,r5    ;r5 to address the input filter data
        move #InputBufLen-1,m5     ;set modulo for input circ buffer

        move #CodecBuf+2,r7        ;for the CODEC's interrupt routine
        move #CodecBufLen*4-1,m7

        move #CodecBuf,r2          ;for "ctrlcd" to initialize the buffer
        move #<4-1,n2
        move #CodecBufLen*4-1,m2

                        ;initialize input/output control words in the buffer
                        ;zero input/output data
      if EVM56K         ;for EVM56002 use MIC input
        ctrlcd  1,r2,CodecBufLen,MIC,LInputGain,RInputGain,LINEO|HEADP,OutputAtten,OutputAtten
      else              ;for DSPCARD4 use LINE input
        ctrlcd  1,r2,CodecBufLen,LINEI,LInputGain,RInputGain,LINEO|HEADP,OutputAtten,OutputAtten
      endif

        opencd SampleFreq/1000.0,HPF     ;start taking samples at given rate

        rts

LastP = *



;*************************************************************************
;Internal data RAM

        LOMEM X:$0000,Y:$0000,L:$0000
        HIMEM X:$00FF,Y:$00FF,L:$00FF

        org     x:user_data
chanflags   dc      $01   ;L+R mix on by default

; Pointer to current filter coeffs and the filter's length (minus 1)
CurFilterCoeffs    dc	    SSBFilter
CurFilterLen    dc	    SSBFilterLen-1

levelCounter dc 0     ;timing counter for input overload LED

	org	y:user_data

;storage area for input audio data
InputBuf  dsm InputBufLen

;*************************************************************************
;External data RAM

        if EVM56K
          LOMEM X:$0200,Y:$0200,L:$0200
          HIMEM X:$3FFF,Y:$3FFF,L:$3FFF
        else
          HIMEM X:$1FFF,Y:$3FFF,L:$1FFF
        endif

      if EVM56K
        org L:$200
      else
        org L:$200
      endif

CodecBuf  dsm     CodecBufLen*4     ;CODEC's input/output buffer

LastL = *
        org X:LastL

	include	'firmacs'	; Handy macros for defining filters

CWFilter
	BasicBandpassFilter	CWFilterLen,CWLowFreq,CWUppFreq

SSBFilter
	BasicBandpassFilter	SSBFilterLen,SSBLowFreq,SSBUppFreq

AMFilter
	BasicBandpassFilter	AMFilterLen,AMLowFreq,AMUppFreq

ThruFilter
	dc	0.99999,0.0	; degenerate all-pass filter of minimum
				; possible length.




