;
; $Header: E:/HAM/EVM/TEST/RCS/SLOWBPSK.ASM 1.9 1997/01/02 20:05:01 dbraun Exp $
;
; $Log: SLOWBPSK.ASM $
; Revision 1.9  1997/01/02 20:05:01  dbraun
; More fixes and improvements to CW ID stuff.
;
; Revision 1.8  1997/01/02 10:12:43  dbraun
; Added CW Tx shape filter coefficients, and fixed bug.
;
; Revision 1.7  1997/01/01 22:03:39  dbraun
; Added CW ID feature.
;
; Revision 1.6  1996/12/29 21:41:05  dbraun
; Added audio monitoring on opposite output channel.
;
; Revision 1.5  1996/12/29 14:32:51  dbraun
; Added Rx carrier control with ^U ^D and ^C characters.
;
; Revision 1.4  1996/12/29 13:35:06  dbraun
; Better freq error LED display.
;
; Revision 1.3  1996/12/29 11:43:01  dbraun
; Added "beat" effect blinking of up/down LEDs to show freq error.
;
; Revision 1.2  1996/12/29 10:02:18  dbraun
; Set levels, etc. and correct LEDs
;
; Revision 1.1  1996/12/27 12:49:41  dbraun
; Initial revision
;
;

;FILE: SLOWBPSK.ASM, Last edition: 2nd of December 1995
;30 (actually 31.25) bps BPSK modem, primary usage is for narrowband QSOs on HF
;(c) 1995 Pawel Jalocha, SP9VRC,
;e-mail: jalocha@chopin.ifj.edu.pl, sp9vrc@gw.sp9kby.ampr.org
;
;This software is not to be used for purposes other than amateur radio
;without a written permision of the author.
;Usage for profit or in commercial/military products is explicitly prohibited.

;This modem uses 7-bit asynchronous ASCII coding and has a terminal-type
;user-interface: whatever is received is being typed on the terminal,
;whatever is typed on the keyboard the modem sends out with the exeption
;of Ctrl-T and Ctrl-R which turns the transmitter on and off.

        opt  mu

        page 132

        nolist
	include 'leonid'
	list

        scsjmp short    ;force short jumps for .if/.endi, .while/.endw, etc.

        title '30 bps BPSK HF modem by SP9VRC'

;*************************************************************************
;compile-time constants

SampleFreq equ 8000.0

AliasFilterLen  equ 32  ;the length of the anti-alias/Hilbert transform filter
AliasLowFreq    equ  400.0 ;lower and upper edges of the input/output filters
AliasUppFreq    equ 2900.0 ;this is about the bassband of my tranceiver

SymbolSepar equ 128     ;Hilbert samples between symbols => 8000/2/SymbolSepar baud
                        ;SymbolSepar should be an even number
                        ;If you change it, there is one place around line 450
                        ;to be manually changed
SymbolShapeLen equ 2*SymbolSepar  ;Symbol's shape length
                        ;For the current symbol's shape (Tr/RxSymbolShape)
                        ;SymbolShapeLen should be 2*SymbolSepar
RxSymbolDecimate equ 4  ;the receiver samples at a lower rate
                        ;not every Hilbert Transform sample

TxCarrier  equ 1800.0   ;[Hz] transmitter carrier frequency
                        ;2nd harmonic is outside of transmitter passband
RxCarrier  equ TxCarrier   ;[Hz] receiver (expected) carrier frequency

RxCarTrack equ 1        ;enable/disable carrier tracking.
                        ;The demodulator is made such that a mis-tune of up
                        ;to at least 5 Hz is tolerable without the need to
                        ;correct the RxCarrier frequency.
                        ;Enable RxCarTrack if you want the demodulator
                        ;to internally track the RxCarrier and
                        ;optionally to steer the receiver via the UP/DOWN
                        ;keys to correct the frequency error at the receiver
                        ;or the Doppler shift resulting from satellite's motion.
RxCarLow   equ RxCarrier-50.0   ;[Hz] lower and upper limits when the carrier is allowed
RxCarUpp   equ RxCarrier+50.0   ;[Hz] to move due to internal tracking
RxCarTrackWeight equ 1.0/16000.0  ;carrier tracking weight per symbol
                        ;Lower numbers make the tracking
                        ;slower but more resistant to noise.
                        ;Higher numbers speed up the tracking
                        ;but make overshoots and the stability gets worse
                        ;in high noise.

InitialRxAutoScan equ 1  ;When you activate RxAutoScan and no carrier is being
                        ;detected the demodulator sweeps its center frequency
                        ;within the range defined below.
RxScanLow equ RxCarrier-25.0    ;As soon as DCD drops the demodulator sweeps the
RxScanUpp equ RxCarrier+25.0    ;carrier within these ranges until the DCD is on again
                        ;Note that searching takes time thus
                        ;wider range => more time to lock onto a signal.
RxScanStep equ   0.1    ;[Hz] scanning step per symbol for auto-scan.
                        ;Larger step => faster search but may overlook
                        ;some signals and make overshoots when locking
                        ;on signals.

RxShowFreqError    equ 1  ; If you want the up/down LEDs to blink at a rate
			  ; proportional to the freq error.
FreqErrorBlinkRate equ 2.0 ; Ratio of freq error to blink rate (must be >= 1.0).

TxAttenuateL     equ  0.0 ;[dB] attenuate the output CODEC's signal.
			  ;Set this between 0.0 and 90.0
			  ;to accomodate the output level for the
			  ;modulator input of your tranceiver.
			  ;Expect some noise out of the CODEC for high
			  ;attenuation levels thus you better use
			  ;an external resistor divider to achieve
			  ;high attenuation factors.
			  ;N1OWU: 34dB is suitable for TS-50 without
			  ;switch-box, 0dB with.
			  ;Both N1OWU transmitters listen to left channel.

TxAttenuateR     equ  0.0 ;N1OWU: The right channel is for audio monitoring

RxGainL          equ  10.0      ;[dB] Rx CODEC's input gain (IC821 on left ch)
                                ;Set this between 0 and 22.5 dB depending on
                                ;the audio level from your tranceiver.

RxGainR          equ  0.0       ;Rx gain for N1OWU's TS-50 on right channel
                                ;Set this between 0 and 22.5 dB depending on
                                ;the audio level from your tranceiver.

RxAGC           equ 0           ;enable the automatic gain adjustment
                                ;for the CODEC's audio input.
                                ;ideally you should avoid the AGC and set
                                ;RxGain for best audio level match.
                                ;Note, that the AGC may get fooled up by
                                ;for example long periods of total silence,
                                ;thus you should rather keep the quelch (if any)
                                ;open all the time.

RxAGC_MSfollow  equ 1.0/256.0   ;update weight per sample
                                ;for the audio Mean Square tracking
RxAGChold       equ 50          ;AGC peak hold time [symbols]
RxAGCfall       equ 5           ;AGC falling period/1.5 dB step [symbols]
RxAGCminMS      equ 0.02        ;min. and max. tolerable audio Mean Square level
RxAGCmaxMS      equ 0.08

RxCorrelFollow  equ 5           ;follow factor for averaging the correlations
                                ;the actuall weight is 1/2^RxCorrelFollow
                                ;per sample.
                                ;Lower number => the demodulator will better
                                ;follow frequency instabilities (phase noise)
                                ;of the receiver and the satellite but Doppler
                                ;tracking and DCD will fluctuate more due
                                ;to noise.
                                ;When you change RxCorrelFollow
                                ;you should change RxProcPipeLen and
                                ;RxCarTrackWeight for optimal perfomance.

RxProcPipeLen   equ 32          ;the samples are stored in this pipe
                                ;and the actuall decoding is delayed
                                ;to give time for the correlations to
                                ;to become stable.

RxDCDdelay      equ 16          ;wait 16 symbols before declaring DCD ON
                                ;to make sure that this is not a momentary
                                ;noise fluctuation
;;;RxDCDhold       equ 128         ;hold DCD for 128 symbols
RxDCDhold       equ 32          ;hold DCD for 32 symbols
                                ;even if there are no signs of carrier

RightTxChannel    equ 0           ;Use the right CODEC's channel not the left.
				  ;N1OWU: Left channel drives both transmitters.

RightRxChannel    equ 1           ;Use the right CODEC's channel not the left.
RightPTTChannel   equ 1           ;Use the right PTT line not the left.
				  ;N1OWU: Right channel is TS-50 HF rig.

BufLen   equ    4*AliasFilterLen ;sample buffer length
                                 ;with some safety margin

SendFixedMessage equ 1          ;transmit one fixed message around the clock
                                ;don't care about the keyboard input

ListenToRxCarrier equ 1		;put RX carrier on opposite channel for testing

;The following are the debugging options...
;normally ALL of them should be set to 0

SPY      equ 0  ; SPY=0 => terminal mode (_all_ SPY_ sub-options must be set to 0 !)
                ; SPY=1 => terminal code excluded, selected debug code included

SPY_RxAudioInp  equ 0   ;the audio input (to check the AGC)
SPY_RxI         equ 0
SPY_RxQ         equ 0
SPY_RxIQ        equ 0   ;I/Q after the Hilbert tranformer + symbol filter
SPY_RxPhase     equ 0   ;phase of the I/Q pair
SPY_RxPhaseDiff equ 0   ;diiferences in phase ( = frequency )
SPY_RxAudioMS   equ 0   ;power (mean square) of the incoming audio
SPY_RxCorrel    equ 0   ;signal power, auto-correl, de-tune, sync.
SPY_RxProcPipePtr equ 0
SPY_RxCrossedIQ equ 0   ;demodulator: crossed succesive vectors
SPY_RxFreqCorr  equ 0   ;demodulator: frequency-error correction vector
SPY_RxCorrIQ    equ 0   ;demodulator: vector after frequency correction
SPY_RxCarFreq   equ 0   ;Rx demodulator carrier frequency
SPY_RxSamplePtr equ 0   ;how the symbol sampling point moves around

SPY_TxSymbolI   equ 0   ;transmitted symbol I-part
SPY_TxCarI      equ 0
SPY_TxFiltOutI  equ 0
SPY_TxOutputI   equ 0   ;transmitted I after the filter and the mixer
SPY_TxAudioOut  equ 0

TxDrift         equ 0   ;enable artificial drift of the transmitter carrier
                        ;works only when SPY=1 and is intended for debugging
                        ;the demodulator in loopback tests.
TxDriftStep     equ 0.01 ;[Hz] drift step per symbol, 0.1*32 = 3.2 Hz/sec
TxDriftLow      equ 1645.0      ;lower and upper limit for the drift
TxDriftUpp      equ 1655.0

MonitorRxAGC    equ 0           ;monitor the gain changes being done by the AGC
                                ;with the UP and DOWN LEDs
                                ;This option conflicts with RxShowFreqError
                                ;when you use the left channel.


;*************************************************************************
;Control characters

LF      equ 10
CR      equ 13
CtrlC   equ 03
CtrlD   equ 04
CtrlR   equ 18
CtrlS   equ 19
CtrlT   equ 20
CtrlU   equ 21
CtrlZ   equ 26

; CW ID character encodings
; The format should be pretty obvious.
; Note that CW_SPC is a special case,
; and that a value of 0 represents the
; end of the string.  

CW_D	equ %1110101000
CW_E	equ %1000
CW_I	equ %101000
CW_K	equ %111010111000
CW_N	equ %11101000
CW_O	equ %11101110111000
CW_P	equ %10111011101000
CW_R	equ %1011101000
CW_S	equ %10101000
CW_T	equ %111000
CW_U	equ %1010111000
CW_W	equ %101110111000
CW_V	equ %101010111000
CW_1	equ %10111011101110111000
CW_STK equ %1110101011101000
CW_SPC equ $800000

;*************************************************************************
;macros

      if RightPTTChannel
PTT     macro mode      ;PTT line: clr/set/chg
        b\mode #4,X:$FFE4
        endm
      else
PTT     macro mode      ;PTT line: clr/set/chg
        b\mode #0,X:$FFE4
        endm
      endif

UpLED   macro mode      ;(a yellow LED connected)
        b\mode #1,X:$FFE4
        endm

DownLED macro mode      ;(a yellow LED connected)
        b\mode #2,X:$FFE4
        endm

YellowLED macro mode    ;(a yellow LED connected)
        b\mode #3,X:$FFE4
        endm

DCDLED  macro mode      ; DCD (green) LED clr/set/chg
        b\mode #13,X:$FFE4
        endm

LongDecayAver macro Input,Aver,Scale
              sub Aver,Input
              rep Scale
                asr Input
              add Input,Aver
              endm

;*************************************************************************
;The actuall code (internal/external program RAM)

        LOMEM P:$0000
        HIMEM P:$1FFF

        org     p:user_code

        jmp <Initialize

;-----------------------------------------------------------------------------

ProcessLoop     ;main loop to process CODEC's samples

RxProcess
        move #>2,a              ;a = 2 (wait for two new samples)
        move X:<CodecPtr,r2     ;after the Hilbert transformer we can decimate
	;N1OWU: why not just use the Leonid "waitblk" macro???
        jsr <WaitSampleBlock    ;but two
        move #BufLen*4-1,m2     ;set m2 so r2 wraps correctly

        move #<4,n2

        ; Save two samples' worth of a mix of both channels, for
        ; audio monitoring.  If you don't want to monitor both
        ; channels, fix this code...
        move X:(r2),x0
        move X:(r2+n2),x1
	move (r2)+
        move X:(r2),a
        add x0,a
        move a1,X:<AudioMonitor1
        move X:(r2+n2),a
        add x1,a
        move a1,X:<AudioMonitor2

	if !RightRxChannel
	  move (r2)-
	endif

	if SPY_RxAudioInp
	  move X:(r2),a
	  jsr <SpyA
	  move X:(r2+n2),a
	  jsr <SpyA
	endif

      if RxAGC                  ;if AGC enabled
        move X:(r2),x0             ;update the mean signal square (MS)
        mpy  x0,x0,a X:(r2+n2),x0  ;of the incoming audio
        macr x0,x0,a X:<RxAudioMS,x1 ;note that here we double the MS
        move a,x0 #RxAGC_MSfollow,y0
        mpy x0,y0,a #1.0-RxAGC_MSfollow,y0
        macr x1,y0,a
        move a,X:<RxAudioMS
       if SPY_RxAudioMS
         jsr <SpyA
       endif
      endif

        move #4*(AliasFilterLen-2),n2
        move #AliasFilterI,r4   ;r4 to address the input filter
        move #<AliasFilterLen-1,m4
        move (r2)-n2            ;move back in the CODEC input buffer
        move #<4,n2             ;n2=4 for easier input adressing
        move r2,x1              ;save r2 (where we start the filter tap)
        clr a X:(r2)+n2,x0 Y:(r4)+,y0
        .loop #AliasFilterLen-1
          mac x0,y0,a X:(r2)+n2,x0  Y:(r4)+,y0
        .endl
        macr x0,y0,a #<AliasFilterQ-AliasFilterI,n4
        move x1,r2              ;restore the tap start
        move (r4)+n4
        clr b X:(r2)+n2,x0 Y:(r4)+,y0
        .loop #AliasFilterLen-1
          mac -x0,y0,b X:(r2)+n2,x0  Y:(r4)+,y0
        .endl
        macr -x0,y0,b           ;a,b = the complex sample (I/Q)

        move ab,L:<RxTmp

        move X:<RxCarPhase,x0  ;get the receiver carrier phase
        move X:<RxCarFreq,a    ;advance the phase
        add x0,a
        move a1,X:<RxCarPhase
        jsr <IQ               ;compute I and Q (modifies a,b,x,y,r0,m0,n0)

        move a,y1
        move L:<RxTmp,x
        mpy x1,y1,a b,y0                        ;mix with the local carrier
        macr x0,y0,a  X:<RxSymbolFilterPtr,r0   ;and pass through
        move #<SymbolShapeLen-1,m0              ;the symbol filter
        mpy x0,y1,b #RxSymbolShape,r4
        macr -x1,y0,b
        move m0,m4
        move ab,L:(r0)+

      if RxSymbolDecimate>1
        move X:<RxSampleTime,a  ;decrement the time counter
        move #>1,x1
        sub x1,a r0,X:<RxSymbolFilterPtr ;did it come to a moment to put out a new bit ?
        move a,X:<RxSampleTime
        jne <TxProcess          ;if not yet, then we avoid this sample
                                ;otherwise we restart the counter
        move #>RxSymbolDecimate,a
        move a,X:<RxSampleTime
      endif

        clr a
        clr b X:(r0),x0 Y:(r4)+,y0
        .loop #SymbolShapeLen-1
          mac x0,y0,a Y:(r0)+,x0
          mac x0,y0,b X:(r0),x0 Y:(r4)+,y0
        .endl
        macr x0,y0,a Y:(r0)+,x0
        macr x0,y0,b r0,X:<RxSymbolFilterPtr

        move ab,L:<RxTmp

      if SPY_RxProcPipePtr
        move r1,a
        rep #8
          asl a
        jsr <SpyA
      endif

      if SPY_RxI
        move X:<RxTmp,a
        jsr <SpyA
      endif

      if SPY_RxQ
        move Y:<RxTmp b
        jsr <SpyA
      endif

      if SPY_RxPhase
        jsr <Phase
        jsr <SpyA
      endif

      if SPY_RxPhaseDiff
        move L:<RxTmp,ab
        jsr <Phase
        move X:<RxPrevPhase,x0
        sub x0,a a,X:<RxPrevPhase
        move #<0,a2
        jsr <SpyA
      endif

      if SPY_RxIQ
        jsr <SpySync
        .if <cc>
          move X:<RxTmp,a
          jsr <SpyA
          move X:<RxTmp,b
          jsr <SpyA
        .endi
      endif

        move X:<RxProcPipePtr,r1
        move #RxProcPipeLen*(SymbolSepar/RxSymbolDecimate)-1,m1
        move L:<RxTmp,ab
        move ab,L:-(r1)          ;put in the most recent I/Q
        move r1,X:<RxProcPipePtr ;and increment the pointer

        move X:<RxCorrelPtr,r4  ;load the correlation pointer
        move #<(SymbolSepar/RxSymbolDecimate)*4-1,m4

        move r4,a               ;if correlation pointer is same as
        move X:<RxSamplePtr,x0  ;the sampling pointer
        eor x0,a1 #<(SymbolSepar/RxSymbolDecimate),n1
        .if <eq>                ;then sample the data bit
          move (r1)-n1
          move L:(r1)-n1,x      ;get the two vectors to be compared
          move L:(r1)+n1,y      ;from the very end of the pipe
          mpy x1,y1,a (r1)+n1
          mac x0,y0,a
          mpy x1,y0,b #<2,n4
          mac -x0,y1,b (r4)+
          jsr <RaughNormIQ
      if SPY_RxCrossedIQ
          move ab,L:<SpyTmp
      endif
          move a,x1
          move b,x0
          move L:(r4)+,a        ;apply frequency correction
          move L:(r4)-n4,b
          jsr <RaughNormIQ
      if SPY_RxFreqCorr
          move ab,L:<SpyTmp
      endif
          move a,y1
          move b,y0
          mpy x1,y1,a ab,L:<RxTmp ;correlate with the expected vector
          mac x0,y0,a            ;save freq. correction for tracking
      if SPY_RxCorrIQ
          mpy -x0,y1,b
          mac x1,y0,b
          move ab,L:<SpyTmp
      endif
      if !SPY                    ;get out the data bit
          btst #7,a2
          jsr <AsyncDecode      ;and pass it through the asynchronous decoder
      endif
      if SPY_RxCrossedIQ||SPY_RxFreqCorr||SPY_RxCorrIQ
          move L:<SpyTmp,ab
          jsr <SpySync
          .if <cc>
            jsr <SpyA
            tfr b,a
            jsr <SpyA
          .endi
      endif

                                ;DCD decision
          move #<(SymbolSepar/RxSymbolDecimate)*4/2,n4
          move L:(r4)+,b        ;b = power
          asr b L:(r4+n4),x     ;x = autocorrelation opposite the symbol
          asr b L:(r4)+,a       ;a = autocorrelation at the symbol
          sub x,a (r4)+         ;a = autocorrelation difference
          sub b,a
          move a1,x0
          move L:(r4)-,a        ;a = spectral asymetry
          rep #5                ;a = 2 * (SymbolSepar/RxSymbolDecimate) * spectral asymetry
            asl a               ;here the multiplication is hardwired
          asl a L:(r4)-,y       ;y = mistune
          sub y,a
          abs a (r4)-
          sub a,b
          or x0,b1 X:<RxDCDcounter,a
          btst #23,b1
          .if <cc>              ;if DCD condition is satisfied
            bset #1,X:<RxState
            move #1.0/RxDCDdelay,x1 ;increment the counter
            add x1,a #0.5,x0
            btst #0,X:<RxState
            move a,X:<RxDCDcounter
            .if <cc>            ;if DCD not yet declared ON
              cmp x0,a          ;check if counter above 1/2
              .if <gt>          ;if so then
                bset #0,X:<RxState      ;declare the DCD as ON
                DCDLED set              ;turn on the DCD LED
	        jsr <StopAutoScan ;deactivate auto scan
              .endi
            .endi
          .else                 ;if DCD condition not satisfied
            bclr #1,X:<RxState
            move #1.0/RxDCDhold,x1
            sub x1,a #-0.5,x0   ;decrement the counter
            btst #0,X:<RxState
            move a,X:<RxDCDcounter
            .if <cs>            ;if DCD is declared ON
              cmp x0,a          ;check if counter below -1/2
              .if <lt>          ;if so then
                bclr #0,X:<RxState      ;declare the DCD as OFF
                DCDLED clr              ;turn off the DCD LED
              .endi
            .endi
          .endi

        if RxCarTrack
          btst #1,X:<RxState      ;DCD condition satisfied ?
          .if <cs>                ;if so then track the carrier
            move L:<RxTmp,ab
            jsr <Phase            ;track the freq. of the incoming signal
            move a,x0 #RxCarTrackWeight,y0
            move X:<RxCarFreq,a
            macr x0,y0,a #4.0*RxCarLow/SampleFreq,x0
            cmp x0,a #4.0*RxCarUpp/SampleFreq,x1  ;keep the carrier frequency
            tlt x0,a                              ;within given limits
            cmp x1,a
            tgt x1,a
            move a,X:<RxCarFreq
          .else                 ;otherwise
	    btst #0,X:<RxState   ;if DCD is declared OFF then
	    .if <cc>
	      btst #0,X:<RxAutoScan   ;auto scan is activated then sweep
	      .if <cs>                ;the carrier within the defined region
		move X:<RxScanStepFreq,b
		tst b X:<RxCarFreq,a
		.if <pl>
		  add b,a #4.0*RxScanUpp/SampleFreq,x0
		  cmp x0,a
		  .if <ge>
		    neg b x0,a
		    move b,X:<RxScanStepFreq
		  .endi
		.else
		  add b,a #4.0*RxScanLow/SampleFreq,x0
		  cmp x0,a
		  .if <le>
		    neg b x0,a
		    move b,X:<RxScanStepFreq
		  .endi
		.endi
		move a,X:<RxCarFreq

;; if RxAutoSCan==0, leave the carrier where it is
;;	     .else                           ;when RxAutoScan==0
;;	       move X:<RxCarFreq,a          ;move the carrier towards nominal
;;	       move #4.0*RxCarrier/SampleFreq,x1
;;	       sub x1,a #4.0*RxScanStep/SampleFreq,y0
;;	       .if <pl>
;;		  sub y0,a
;;		  .if <lt>
;;		    clr a
;;		  .endi
;;	       .else
;;		  add y0,a
;;		  .if <gt>
;;		    clr a
;;		  .endi
;;	       .endi
;;	       add x1,a
;;	      move a,X:<RxCarFreq

	    .endi  ;of RxAutoScan
           .endi
          .endi
        endif   ;of RxCarTrack

      if SPY_RxCarFreq
          move X:<RxCarFreq,a
          move #4.0*RxCarrier/SampleFreq,x0
          sub x0,a
          jsr <SpyA
      endif

        .endi   ;if we are at the symbol sampling moment

        move #<(SymbolSepar/RxSymbolDecimate)*4/2,n4
        move X:<RxSamplePtr,x0
      if SPY_RxSamplePtr
        tfr x0,a #>RxCorrel,y0
        sub y0,a
        rep #8
          asl a
        jsr <SpyA
      endif
        lua (r4)+n4,r5
        move m4,m5
        move r5,a        ;if sampling pointer is oposite the correlation
        eor x0,a1 #<4,n4 ;pointer then adjust the sampling point
        .if <eq>
          move n4,n5
          move (r4)+
          move (r5)+
          move L:(r4)-n4,y
          move L:(r5)-n5,a
          sub y,a L:(r4)+n4,y
          move L:(r5)+n5,b
          sub y,b
          cmp a,b
          .if <gt>
            move (r5)-n5
            move (r5)-
            move r5,X:<RxSamplePtr
          .else
            move L:(r4+n4),y
            move L:(r5+n5),b
            sub y,b
            cmp a,b
            .if <gt>
              move (r5)+n5
              move (r5)-
              move r5,X:<RxSamplePtr
            .endi
          .endi
          move (r4)-
        .endi

        move #<(SymbolSepar/RxSymbolDecimate)/2,n1 ;update correlations which are used
        move #<RxCorrel0-RxCorrel,n4    ;for freq. and timing corrections
                                        ;and DCD decisions
        move X:(r1),x0                  ;signal power
        mpy x0,x0,a Y:(r1),x0
        mac x0,x0,a
        jsr <LowPass2el

        move L:(r1)+n1,y                ;autocorrelation at SymbolSepar
        move L:(r1+n1),x
        mpy x1,y1,a (r1)-n1
        mac x0,y0,a
        move a,L:<RxTmp
        .if <mi>
          neg a
        .endi
        jsr <LowPass2el

        move L:(r1)+n1,y
        move L:(r1+n1),x
        mpy x1,y0,a (r1)-n1     ;frequency error
        mac -x0,y1,a
        btst #23,X:<RxTmp
        .if <cs>
          neg a
        .endi
        jsr <LowPass2el

                                ;spectral asymetry
        move L:(r1)+,y
        move L:(r1)-,x
        mpy x1,y0,a
        mac -x0,y1,a
        jsr <LowPass2el

        move r4,X:<RxCorrelPtr

        if SPY_RxCorrel
          move X:<RxProcPipePtr,a1
          move #>RxProcPipe,x0
          eor x0,a1 #RxCorrel,r4
          .if <eq>
            jsr <SpySync
            .if <cc>
              move #<4,n4
              move X:<RxCarFreq,a
              move #4.0*RxCarrier/SampleFreq,x0
              sub x0,a
              jsr <SpyA
              .loop #4
                .loop #(SymbolSepar/RxSymbolDecimate)
                  move L:(r4),a
                  rep #4
                    asl a
                  jsr <SpyA
                  move (r4)+n4
                .endl
                clr a
                jsr <SpyA
                move (r4)+
              .endl
              jsr <SpyA
            .endi
          .endi
        endif

	if RxShowFreqError&&(!MonitorRxAGC)

	  move X:<RxCarFreq,a
	  move #4.0*RxCarrier/SampleFreq,x0
	  sub x0,a 	; Calculate Rx carrier frequency error
	  move a,x1

	  asl a		; Multiply by 4 so that beat freq is correct.
	  asl a		; Watch for potential overflow here!

	  if FreqErrorBlinkRate>1.0
	    move #1.0/FreqErrorBlinkRate,x0  ;scale as desired
	    move a,y0
	    mpyr x0,y0,a
          endif

	  move X:<RxCarFreqErrorPhase,x0
	  add x0,a
	  move a1,X:<RxCarFreqErrorPhase	;advance phase of error "beat".

	  btst #23,a1	;check sign of error "beat" phase
	  .if <cc>
	    UpLED clr	;if positive, turn off LEDs
	    DownLED clr
	  .else
	    btst #23,x1	;check sign of carrier freq error
	    .if <cs>
	      DownLED set	;if negative, turn on "down" LED
	    .else
	      UpLED set
	    .endi
	  .endi
	endif ; of RxShowFreqError

;Here comes the transmitter part
;we begin with getting the digital data to be sent out
;and generate the I/Q values for this data

TxProcess
        move X:<TxBitTime,a     ;decrement the bit time counter
        move #>1,x1
        sub x1,a #<0,b          ;did it come to a moment to put out a new bit ?
        move a,X:<TxBitTime     ;if not yet, then we don't put out anything
        jne <TxMixOut           ;if not yet, then we don't put out anything
                                ;otherwise we restart the counter
          move #>SymbolSepar,a  ;and put out the new bit
          move a,X:<TxBitTime
      if RxAGC
         jsr <RxCheckAudioMS
      endif
      if SPY                    ;in SPY mode send continues stream of random data
          if TxDrift                    ;Drift the Tx carrier to simulate
            move X:<TxDriftStepFreq,b   ;Doppler drift
            tst b X:<TxCarFreq,a
            .if <pl>
              add b,a #4.0*TxDriftUpp/SampleFreq,x0
              cmp x0,a
              .if <ge>
                neg b x0,a
                move b,X:<TxDriftStepFreq
              .endi
            .else
              add b,a #4.0*TxDriftLow/SampleFreq,x0
              cmp x0,a
              .if <le>
                neg b x0,a
                move b,X:<TxDriftStepFreq
              .endi
            .endi
            move a,X:<TxCarFreq
          endif
          jsr <Rand48
          btst #10,a1
          .if <cs>
            clr b #<0.5,a
          .else
            clr b #<-0.5,a
          .endi
       else                     ;in non-Spy mode get data from async. encoder
	  jsr <GetCwBit
	  ;if GetCwBit returns w/ zero flag set, there are no more
	  ; CW bits, so use a bit from AsyncDecode instead.
          .if <eq>
	    jsr <AsyncEncode
	    move X:<TxDigLevel,a
	    .if <cs>               ;invert the phase for one-bits
	      neg a
	    .endi
	    clr b a,X:<TxDigLevel  ;set Q=0 (we only do BPSK)
	    move #TxSymbolShape,r4
	  .else
	    .if <cs>
	      clr b #<0.5,a	;send a carrier for a dot or dash
	    .else
	      clr b #<0.0,a     ;send nothing for space
	    .endi
	    move #TxCwSymbolShape,r4
	  .endi
	  PTT tst
	  jcc <TxMixOut         ;don't send out anything when PTT==0
       endif

        if SPY_TxSymbolI
          jsr <SpyA
        endif

        ; Above we have set r4 to point to the correct shape coefficients.
        move X:<TxSymbolFilterPtr,r0    ;overlap the next symbol
        move #<SymbolShapeLen-1,m0      ;with amplitudes A(I) and B(Q)
        move m0,m4
        move a,x1
        move b,x0
        .loop #SymbolShapeLen
          move X:(r0),a Y:(r4)+,y0
          macr x1,y0,a Y:(r0),b
          macr x0,y0,b a,X:(r0)
          move b,Y:(r0)+
        .endl

        if SPY_TxFiltOutI
          move X:(r0),a
          jsr <SpyA
        endif

TxMixOut
        move X:<TxCarPhase,x0  ;get the transmitter carrier phase
        move X:<TxCarFreq,a    ;advance the phase
        add x0,a X:<TxSymbolFilterPtr,r1
        move #<SymbolShapeLen-1,m1
        move a1,X:<TxCarPhase
        jsr <IQ               ;compute I and Q (modifies a,b,x,y,r0,m0,n0)
        if SPY_TxCarI
          jsr <SpyA
        endif
        move a,y1
        clr a L:(r1),x
        move a,L:(r1)+
        mpy x1,y1,a b,y0      ;mix with the carrier
        macr -x0,y0,a
        mpy x0,y1,b r1,X:<TxSymbolFilterPtr
        macr x1,y0,b

        if SPY_TxOutputI
          jsr <SpyA
        endif

TxOutput                        ;a/b = I/Q to be send out
        move X:<CodecPtr,r2
        move #BufLen*4-1,m2

        move #AliasFilterI,r4
        move #<AliasFilterLen-1,m4
        move #<AliasFilterQ-AliasFilterI,n4
        move (r2)+
      if RightTxChannel
        move (r2)+
      endif
        move #4*(AliasFilterLen-2),n2
        move a,y1
        move b,y0
        move (r2)-n2            ;move back in the CODEC input buffer
        move #<4,n2             ;n2=4 for easier input adressing
      if SPY_TxAudioOut
          nop
          move (r2)-n2
          move (r2)-n2
          move Y:(r2)+n2,a
          jsr <SpyA
          move Y:(r2)+n2,a
          jsr <SpyA
      endif

      ; N1OWU: If transmitting, add the audio output to the monitoring
      ; output and stick it (two samples) in the output channel used for
      ; monitoring (the opposite of that used for transmitting).
      ; This code could surely be simplified...
          nop
          move (r2)-n2
          move (r2)-n2
	  if RightTxChannel
	    move Y:(r2),a
	    move X:<AudioMonitor1,x0
	    add x0,a
	    move (r2)-
	    move a1,Y:(r2)+
	    move (r2)+n2

	    move Y:(r2),a
	    move X:<AudioMonitor2,x0
	    add x0,a
	    move (r2)-
	    move a1,Y:(r2)+n2
	    move (r2)+
	  else
	    move Y:(r2),a
	    move X:<AudioMonitor1,x0
	    add x0,a
	    move (r2)+
	    move a1,Y:(r2)-
	    move (r2)+n2

	    move Y:(r2),a
	    move X:<AudioMonitor2,x0
	    add x0,a
	    move (r2)+
	    move a1,Y:(r2)-
	    move (r2)+n2
	  endif

        .loop #AliasFilterLen-2
          move Y:(r4),x0
          move Y:(r2),a
          mac x0,y1,a Y:(r4+n4),x0
          macr -x0,y0,a (r4)+
          move a,Y:(r2)+n2
        .endl
        .loop #2
          move Y:(r4),x0
          mpy x0,y1,a Y:(r4+n4),x0
          macr -x0,y0,a (r4)+
          move a,Y:(r2)+n2
        .endl
        move (r2)-
      if RightTxChannel
        move (r2)-
      endif
        move r2,X:<CodecPtr

        jmp <ProcessLoop

;-----------------------------------------------------------------------------

CheckSampleBlock        ;a = how many samples we want
        asl a           ;r2 = where we wait for the block to start
        asl a #>2,x1
        add x1,a
        move a,x1
CheckSampleBlock_1
        move r7,a
        move r2,x0
        sub x0,a
        jpl <CheckSample_cmp
          move #>BufLen*4,x0
          add x0,a
CheckSample_cmp
        cmp x1,a
        rts             ;on output: a,x0,x1 are modified
                        ;If carry=0 => we are sure that:
                        ;1. there are at least A new input samples at X:(r2)
                        ;   to be read
                        ;2. there are at least A locations at Y:(r2) to
                        ;   be written


WaitSampleBlock         ;wait for a block of samples from the CODEC
                        ;on input: a = how many samples we want
                        ;          r2 = where the block starts
        jsr <CheckSampleBlock
        jcc <WaitSample_ret
WaitSample_loop
        wait            ;wait (at low power) until an interrupt comes
        jsr <CheckSampleBlock_1
        jcs <WaitSample_loop
WaitSample_ret
        rts             ;on output: a,x0,x1 are modified
                        ;We are sure that:
                        ;1. there are at least A new input samples at X:(r2)
                        ;   to be read
                        ;2. there are at least A locations at Y:(r2) to
                        ;   be written


;-----------------------------------------------------------------------------

        if RxAGC

RxCheckAudioMS                  ;check the audio level and adjust
                                ;the CODEC's input gain if needed
      if MonitorRxAGC
        UpLED clr
        DownLED clr
      endif

        move X:<RxAudioMS,a
        move #BufLen*4-1,m2     ;r2/m2 must point to the CODEC's buffer
        move #RxAGCminMS,x0     ;see, if the MS is below the minimal level
        cmp x0,a X:<CodecPtr,r2
        jcc <CheckMSmax         ;if not, then check if above the max. level
                                ;if so, increase the CODEC's input gain
        move X:<RxAGCcount,a    ;decrement the timeout
        move #>1,x0
        sub x0,a 
        move a,X:<RxAGCcount
        jgt <CheckMS_OK         ;leave if not yet zero
        jsr <RxGainUp           ;increase the CODEC's gain
        jcs <CheckMS_OK
        move #0.7071,y0         ;increase the MS to follow
        move X:<RxAudioMS,x0    ;the gain change faster
        mpyr x0,y0,a #>RxAGCfall,x0 ;preset the AGC "timer" to prohibit
        asl a x0,X:<RxAGCcount      ;gain increase for the RxAGCfall period
        move a,X:<RxAudioMS
      if MonitorRxAGC
        UpLED set
      endif
        jmp <CheckMS_OK

CheckMSmax                      ;is the MS above the given maximum
        move #>RxAGChold,y0     ;preset the AGC "timer" to prohibit gain increase
        move #RxAGCmaxMS,x0     ;for the RxAGChold period
        cmp x0,a y0,X:<RxAGCcount ;MS above maximum ?
        jcs <CheckMS_OK         ;if not then we are done
GainDown
        jsr <RxGainDown         ;otherwise decrease the CODEC's gain
        jcs <CheckMS_OK
        move #0.7071,y0         ;decrease the MS to follow expected
        move X:<RxAudioMS,x0    ;gain reduction faster
        mpyr x0,y0,a #>RxAGChold,x0     ;initialize the AGC hold count-down
        move a,X:<RxAudioMS
        move x0,X:<RxAGCcount
      if MonitorRxAGC
        DownLED set
      endif

CheckMS_OK
        rts

RxGainUp                        ;increase the CODEC's input gain (both channels)
        clr a #>$0F0F00,x0      ;r2/m2 should point to the CODEC's buffer
        move Y:(r2),a1          ;get the CODEC's input control word
        and x0,a                ;extract the gain bits
	cmp x0,a  #>$010100,x0  ;already maximum ?
        jeq <RxGainChange_abort ;if so then abort this attempt
	add x0,a  #>$F0F000,x0  ;if not, increment the gain by 1
	move a1,x1
	move Y:(r2),a1          ;and reload all the control words
	and x0,a  n2,x0         ;in the output buffer
	or x1,a  #<4,n2         ;make n2=4 for a moment
	.loop #BufLen
	  move a1,Y:(r2)+n2
	.endl
	move x0,n2              ;restore n2
        andi #$FE,ccr           ;clear carry
        rts                     ;modifies a,x0,x1

RxGainDown                      ;decrease the CODEC's input gain (both channels)
        clr a #>$0F0F00,x0      ;r2/m2 should point to the CODEC's buffer
        move Y:(r2),a1          ;get the CODEC's input control word
        and x0,a  #>$010100,x0  ;extract the gain bits
        sub x0,a  #>$F0F000,x0  ;attempt to decrease the gain
        jcs <RxGainChange_abort ;jump if overflow
        move a1,x1
        move Y:(r2),a1          ;reload all the input control words
        and x0,a  n2,x0         ;in the buffer with the new input gain
        or x1,a  #<4,n2         ;n2=4 for a moment
        .loop #BufLen
          move a1,Y:(r2)+n2
        .endl
        move x0,n2              ;restore n2
        andi #$FE,ccr           ;clear carry
        rts                     ;modifies a,x0,x1

RxGainChange_abort
        ori #$01,ccr            ;set carry to indicate that we couldn't
        rts                     ;change the gain

        endif

;-----------------------------------------------------------------------------

LowPass2el ;a = input, r4,r4+n4 = filter storage (2 elements)
        move L:(r4+n4),b  ;b = E
        move L:(r4),x   ;x = F
        add x,b         ;b = E+F
        asr b           ;b = (E+F)/2
        sub b,a L:(r4+n4),b ;a = I - (E+F)/2, b = E
        rep #RxCorrelFollow-1
          asr a         ;a = (I - (E+F)/2)/16
        add a,b L:(r4+n4),a ;b = new E, a = old E
        sub x,a b,L:(r4+n4) ;save new E, a = E - F
        rep #RxCorrelFollow
          asr a
        add x,a         ;a = F + (E-F)/32
        move a,L:(r4)+  ;save new F
                        ;a = output, r4 moved by 1, x,b modified
        rts             ;

;-----------------------------------------------------------------------------

PI      equ     3.14159265358979323846

;this routine computes a cosine/sine pair using the sine ROM
;with a second order (linear+quadrature) approximation between table points
IQ                              ;x0 = angle ( -1 = -PI, +1 = +PI)
        move #>$80,x1   ;shift out 8 most significant bits
        mpy x0,x1,a  #>$FF,x0
        move x0,m0
        and x0,a     #>$100,x0
        or x0,a      #<$40,n0
        move a1,r0      ;put the 8 most significant bits into r0 with offset = $100
        move a0,y0      ;save the remaining bits in y0
        jclr #23,y0,SinTable_lev2
          move (r0)+
SinTable_lev2
        move Y:(r0+n0),x0       ;x0 = coarse cosine
        move Y:(r0),x1          ;x1 = coarse sine
        mpyr x1,y0,a  #PI/256.0,y1
        tfr x0,a  a,x1
        macr -x1,y1,a           ;a = fine cosine
        mpyr x0,y0,b  Y:(r0),x1
        tfr x1,b  b,x1
        macr x1,y1,b  #PI*PI/2.0/65536.0,y1  ;b = fine sine
        mpyr y0,y0,a  a,x0
        move a,y0
        mpyr y0,y1,a
        tfr x0,a  a,y1
        macr -x0,y1,a  b,x1     ;a = super fine cosine
        macr -x1,y1,b           ;b = super fine sine
        rts                     ;x,y are modified
                                ;r0,m0,n0 are modified
                                ;maximum error is about 0.7E-6
                                ;execution time 4+64+4 clock cycles
                                ;including "jsr <IQ_lev2" and "rts"

RaughNormIQ             ;scale an I/Q pair such that the larger number
                        ;is within the range +/-0.5..1.0
        cmpm a,b        ;which number is greater ?
        jge <NormAftB   ;if b then follow the normalization according to it
        tst a           ;otherwise follow a
        jes <RightA     ;if too big (extension in use) the shift to the right
        jnr <DoneA      ;if normalized then we are done
LeftA     asl b         ;otherwise shift left both numbers
	  asl a
        jnn <LeftA      ;until A is normalized
        rts
RightA    asr b         ;if extension set then shift both number to the right
	  asr a
        jes <RightA     ;until a is normalized
DoneA   rts

NormAftB                ;same stuff for b
	tst b
	jes <RightB
        jnr <DoneB
LeftB	  asl a
	  asl b
	jnn <LeftB
        rts
RightB	  asr a
	  asr b
	jes <RightB
DoneB   rts


;The following routine computes the phase of the given I/Q pair
;this routine is not very precise: maximum error is 2E-4 (2E-4/PI radians)
;the error could be made lower but I required that the calculation
;is exact for PI/4, PI/2, PI*3/4, etc.
Phase      ;a = I-part (24 bits), b = Q-part (24 bits)
           ;-1.0 <= a < 1.0, -1.0 <= b < 1.0
        tst b #<0,x0    ;if Q-part negative => turn by PI
        .if <mi>
          neg a #<%10000000,x0
          neg b
        .endi
        tst a           ;if I-part negative => turn by -PI/2
        .if <mi>
          bset #22,x0
          tfr a,b b,a   ;swap I with Q
          neg b         ;negate the new Q
        .endi
        add b,a                 ;add Q to I (what if I+Q >= 1.0 ?)
        asr a                   ;I+Q /= 2 in case it is > 1
        asr b a,x1              ;Q /= 2 because of the above, x1 = I+Q
        andi #$FE,ccr           ;divive Q by (I+Q)
        rep #24
          div x1,b
        tfr x0,a b0,x0          ;x0 = Q div (I+Q), a=approx angle
        tfr x0,b #0.5,x0        ;b = Q/(I+Q)
        sub x0,b #0.25,x0       ;b = Q/(I+Q) - 1/2 =: X
        add x0,a b,x1 #0.63361654,y1 ;angle += 0.25, x1 = X
        mac x1,y1,a x1,x0       ;angle += F1*X, x0 = X
        mpyr x0,x0,b #-0.73728217,y1 ;b = X*X
        mpyr x1,y1,b b,x0       ;b = F3*X, x0 = X*X
        move b,y1
        mac x0,y1,a x0,y0       ;angle += F3*X * X*X, y0 = X*X
        mpyr x0,x0,b #0.85568791,y1 ;b = (X*X)*(X*X)
        mpyr x1,y1,b b,x0       ;b = F5*X, x0 = X*X*X*X
        move b,y1
        mac x0,y1,a             ;angle += F5*X * X*X*X*X
        mpyr x0,y0,b #-0.17769569,y1 ;b = (X*X*X*X)*(X*X)
        mpyr x1,y1,b b,x0       ;b = F5*X, x0 = X*X*X*X*X*X
        move b,y0
        macr x0,y0,a            ;angle += F5*X * X*X*X*X*X*X
        rts             ;a = the phase (-1.0 => -PI, 1.0 => PI)
                        ;b,x,y are modified
                        ;execution time is about 130 cycles (65 instructions)


;-----------------------------------------------------------------------------

      if SPY
Rand48                          ;a simple random number generator
        move L:<Rand,a
        asl a L:<RandInc,x
        adc x,a
        move a10,L:<Rand
        rts                     ;returns a pseudo-random number in a
                                ;modifies x
      endif
;-----------------------------------------------------------------------------
;Asynchronous decoder (1 start bit, 7 data bits, 1 stop bit)
;By the way: do we need the stop bit ?

      if !SPY
AsyncDecode             ;carry = newly received bit
        move X:<AsyncDecodeShiftReg,a1
        ror a1 X:<AsyncDecodeBitCount,b ;move this bit into the shift register
        tst b a1,X:<AsyncDecodeShiftReg
        .if <eq>        ;if counter==0 (previous character was completed)
          btst #23,a1
          .if <cc>      ;if new bit == 0 => another character is coming
            move #>8,b  ;set counter to 8 (we nee to receive 8 more bits)
            move b,X:<AsyncDecodeBitCount
          .endi         ;otherwise no action (line stays idle)
        .else
          move #>1,x0   ;decrement the bit counter
          sub x0,b #>$7F,x0
          move b,X:<AsyncDecodeBitCount
          .if <eq>      ;counter==0 => this was the last bit
            rep #24-8
              lsr a1
            and x0,a1
            jset #0,X:<RxState,SendChar ;if carrier is present
          .endi                         ;send the new character to terminal
        .endi
        rts             ;modifies a,b,x,y,r0/m0

SendChar        ;send a character in a1 to terminal
        move a1,x0
        move #<CR,a1        ;is this CR ?
        eor x0,a1
        .if <eq>            ;if it is then send LF afterwards
           putc
           move #>LF,x0
           putc
        .else
           putc
        .endi
        rts

AsyncEncode
        move X:<AsyncEncodeBitCount,b
        tst b X:<AsyncEncodeShiftReg,a1
        .if <eq>
          jsr <ReadChar
          .if <cc>
            move #>8,b
            move b,X:<AsyncEncodeBitCount
            bset #7,a1
            move a1,X:<AsyncEncodeShiftReg
            andi #$FE,ccr
;          .else
;            ori #$01,ccr
          .endi
        .else
          move #>1,x0
          sub x0,b
          lsr a1 b,X:<AsyncEncodeBitCount
          move a1,X:<AsyncEncodeShiftReg
        .endi
        rts     ;carry = next bit to be transmitted

ReadChar
      if SendFixedMessage
        move #$1FFFF,m0
        move X:<FixedMessagePtr,r0
        clr a
        move Y:(r0)+,x0
        eor x0,a
        .if <eq>
          move #FixedMessage,r0
          nop
          move Y:(r0)+,x0
        .endi
        move r0,X:<FixedMessagePtr
        andi #$FE,ccr
      else
        lookc 0
      endif
        .if <cc>
          move #<CtrlT,a1	; Ctrl-T for transmitting.
          eor x0,a1
          jeq <PTT_on

          move #<CtrlR,a1	; Ctrl-R for receiving.
          eor x0,a1
          jeq <PTT_off

          move #<CtrlS,a1	; Ctrl-S for Rx carrier scan until locked.
          eor x0,a1
          jeq <StartAutoScan

          move #<CtrlC,a1	; Ctrl-C to reset carrier to nominal
          eor x0,a1
          jeq <ResetFreq

          move #<CtrlU,a1	; Ctrl-U to increase the Rx carrier freq
          eor x0,a1
          jeq <ManualFreqUp

          move #<CtrlD,a1	; Ctrl-D to decrease the Rx carrier freq
          eor x0,a1
          jeq <ManualFreqDown

          move #<CtrlZ,a1	; Ctrl-Z to send CW ID
          eor x0,a1
          jeq <SendCwId

          move x0,a1
          andi #$FE,ccr
        .endi
        rts     ;carry = 0 if new character has been read from the terminal
                ;a1 = new character

PTT_on  PTT set
        ori #$01,ccr
        rts

PTT_off PTT clr
        ori #$01,ccr
        rts

StartAutoScan
	bset #0,X:<RxAutoScan   ;activate auto scan
        ori #$01,ccr
	rts

StopAutoScan
	bclr #0,X:<RxAutoScan   ;deactivate auto scan
	clr a
	move a0,X:<RxCarFreqErrorPhase
        ori #$01,ccr
	rts

ManualFreqUp
        move X:<RxCarFreq,a    ;increase the Rx carrier freq by 1Hz
	move #>4.0/SampleFreq,x0
        add x0,a
        move a,X:<RxCarFreq
	jmp <StopAutoScan ;deactivate auto scan and return

ManualFreqDown
        move X:<RxCarFreq,a    ;decrease the Rx carrier freq by 1Hz
	move #>-4.0/SampleFreq,x0
        add x0,a
        move a,X:<RxCarFreq
	jmp <StopAutoScan ;deactivate auto scan and return

ResetFreq
	move #4.0*RxCarrier/SampleFreq,a
        move a,X:<RxCarFreq
	if RxShowFreqError
	  UpLED clr
	  DownLED clr
	endif
	jmp <StopAutoScan ;deactivate auto scan and return

SendCwId
	; Activate the GetCwBit subroutine.
	move #CwIdString,a0
	move a0,X:<CwWordPtr
	move #1,a0
	move a0,X:<CwCount
        ori #$01,ccr
	rts

      endif


GetCwBit
	; This returns the next symbol for CW output.  Each 1 or 0 bit in
	; the CwId string becomes two symbols.  This gives about a 14 WPM
	; speed.  To double the speed, the code can be easily changed.

	move X:<CwWord,a
	move X:<CwCount,b0
	btst #0,b0		;remember that each bit is sent twice...
	.if <cs>
	  asl a
	.endi
	dec b
	.if <eq>
	  move X:<CwWordPtr,r0
	  nop
	  move Y:(r0)+,a	;get the next cw character word
	  tst a			;test for zero, which means no more characters
	  .if <eq>
	    rts		;return w/ zero flag set to mean no more bits
    	  .endi
	  move r0,X:<CwWordPtr	;advance the word pointer
    	  btst #23,a1		;bit 23 set means special word-space word
    	  .if <cs>
    	    clr a
    	    move #2*3+1,b0	;initialize word and count for word-space
	  .else
	    move #2*24+1,b0		;prepare to normalize the word
CwNormalizeLoop
	    btst #23,a1		;shift the word left and decrement count
	    jcs <CwNormalizeEnd ;until bit 23 is 1.
	    dec b
	    btst #0,b0		;remember that each bit is sent twice...
	    .if <cs>
	      asl a
	    .endi
	    jmp <CwNormalizeLoop
CwNormalizeEnd
	  .endi
	.endi
	move a1,X:<CwWord	;save the updated word
	move b0,X:<CwCount	;save the updated count
	btst #23,a1		;return bit in carry flag
        andi #$fb,ccr  		;clear zero flag
	rts



;-----------------------------------------------------------------------------


;-----------------------------------------------------------------------------

Initialize      ;initialize registers, buffers, windows, etc.

        ori #%00000100,omr      ;enable the sine ROM table
                                ;this is for the IQ routine

        clr a #RxProcPipe,r0
        move #RxProcPipeLen*(SymbolSepar/RxSymbolDecimate)-1,m0
        rep #RxProcPipeLen*(SymbolSepar/RxSymbolDecimate)
          move a,L:(r0)+

        clr a #RxSymbolFilterTap,r0
        move #SymbolShapeLen-1,m0
        rep #SymbolShapeLen
          move a,L:(r0)+
        clr a #TxSymbolFilterTap,r0
        rep #SymbolShapeLen
          move a,L:(r0)+

        clr a #RxCorrel,r0
        move #(SymbolSepar/RxSymbolDecimate)*4-1,m0
        rep #(SymbolSepar/RxSymbolDecimate)*4
          move a,L:(r0)+
        clr a #RxCorrel0,r0
        rep #(SymbolSepar/RxSymbolDecimate)*4
          move a,L:(r0)+

        move #Buffer+2,r7        ;for the CODEC's interrupt routine
        move #BufLen*4-1,m7

        move #Buffer,r2          ;for "cdctrl" to initialize the buffer
        move #<4-1,n2
        move #BufLen*4-1,m2
                        ;initialize input/output control words in the buffer
                        ;zero input/output data
      if EVM56K         ;for EVM56002 use MIC input
        ctrlcd  1,r2,BufLen,MIC,RxGainL,RxGainR,LINEO|HEADP,TxAttenuateL,TxAttenuateR
      else              ;for DSPCARD4 use LINE input
        ctrlcd  1,r2,BufLen,LINEI,RxGainL,RxGainR,LINEO|HEADP,TxAttenuateL,TxAttenuateR
      endif
        opencd SampleFreq/1000.0,HPF     ;start taking samples at given rate

        jmp <ProcessLoop

        if SPY

SpySync move a10,L:<SpySave     ;output: carry=1 => no spy request
        move a2,X:<SpySave+1    ;carry=0 => spy request !
        move x0,Y:<SpySave+1    ;512 words (jsr <SpyA) should follow
        move x1,Y:<SpyCount
        move X:<SpyCount,a
        tst a
        jne <SpyCont
        lookc 0
        jcs <Spy_end
        move #>'S',a
        cmp x0,a
        ori #$01,ccr
        jne <Spy_end
        move #>'P',x0
        putc
        move #>0,x0      ; send a 0 meaning non-complex data
        putc
        move #>512,a
        move a,X:<SpyCount
SpyCont andi #$FE,ccr
        jmp <Spy_end

SpyA    move a10,L:<SpySave
        move a2,X:<SpySave+1
        move x0,Y:<SpySave+1
        move x1,Y:<SpyCount
        move X:<SpyCount,a
        tst a
        jne <Spy_copy

Spy_check
        lookc 0
        jcs <Spy_end
        move #>'S',a
        cmp x0,a
        jne <Spy_end
        move #>'P',x0
        putc
        move #>0,x0      ; send a 0 meaning non-complex data
        putc
        move #>512,a
Spy_copy
        move #>1,x0
        sub x0,a
        move a,X:<SpyCount

        move X:<SpySave,a
        rep #8
          lsr a
        move a1,x0
	putc
        move X:<SpySave,a
        rep #16
          lsr a
        move a1,x0
        putc

Spy_end move L:<SpySave,a10
        move X:<SpySave+1,a2
        move Y:<SpySave+1,x0
        move Y:<SpyCount,x1
        rts

        endif

;*************************************************************************
;Internal data RAM

        LOMEM X:$0000,Y:$0000,L:$0000
        HIMEM X:$00FF,Y:$00FF,L:$00FF

        org L:user_data

        if SPY
SpySave dc 0,0
SpyCount dc 0
SpyTmp   ds 1
        endif

Rand    dc $954820584783        ;seed and increment for the random
RandInc dc $894659236475        ;number generator

RxTmp      ds 1

LastL = *
        org X:LastL
        org Y:LastL

        org X:
CodecPtr    dc Buffer

RxCarPhase  dc 0
RxCarFreq   dc 4.0*RxCarrier/SampleFreq
RxScanStepFreq dc 4.0*RxScanStep/SampleFreq

TxCarPhase  dc 0
TxCarFreq   dc 4.0*TxCarrier/SampleFreq

RxSymbolFilterPtr dc RxSymbolFilterTap
TxSymbolFilterPtr dc TxSymbolFilterTap

TxDigLevel dc 0.5

RxState dc 0            ;bit #0 = DCD status
                        ;bit #1 = DCD conditiion momentary state
RxDCDcounter dc 0       ;DCD acceptance counter

        if RxSymbolDecimate>1
RxSampleTime dc RxSymbolDecimate
        endif

RxSamplePtr dc RxCorrel
RxCorrelPtr dc RxCorrel
RxProcPipePtr dc RxProcPipe

TxBitTime dc SymbolSepar

RxAutoScan dc InitialRxAutoScan

AudioMonitor1 dc 0
AudioMonitor2 dc 0

CwWord	   dc 0
CwCount	   dc 1
CwWordPtr  dc CwIdStringEnd

        if RxShowFreqError
RxCarFreqErrorPhase dc 0
        endif

        if RxAGC
RxAudioMS   dc 0
RxAGCcount  dc RxAGCfall
        endif

        if !SPY
AsyncEncodeShiftReg dc 0
AsyncEncodeBitCount dc 0
AsyncDecodeShiftReg dc 0
AsyncDecodeBitCount dc 0
        endif

        if SendFixedMessage
FixedMessagePtr dc FixedMessage
        endif

        if SPY_RxPhaseDiff
RxPrevPhase dc 0
        endif

        if SPY&&TxDrift
TxDriftStepFreq dc 4.0*TxDriftStep/SampleFreq
        endif

        org X:
LastX = *
        org Y:
LastY = *
        if @cvi(LastX)>=@cvi(LastY)
          org L:LastX
        else
          org L:LastY
        endif

;*************************************************************************
;External L/X/Y data RAM

        if EVM56K
          LOMEM X:$2000,Y:$2000,L:$2000
          HIMEM X:$3FFF,Y:$3FFF,L:$3FFF
        else
          LOMEM X:$0200,Y:$0200,L:$0200
          HIMEM X:$1FFF,Y:$1FFF,L:$1FFF
        endif

      if EVM56K
        org L:$2000
      else
        org L:$200
      endif

RxSymbolFilterTap dsm SymbolShapeLen
TxSymbolFilterTap dsm SymbolShapeLen

RxCorrel  dsm (SymbolSepar/RxSymbolDecimate)*4
RxCorrel0 dsm (SymbolSepar/RxSymbolDecimate)*4

Buffer  dsm BufLen*4    ;CODEC's input/output buffer

RxProcPipe dsm RxProcPipeLen*(SymbolSepar/RxSymbolDecimate)

LastL = *
        org X:LastL
        org Y:LastL

        org X:
LastX = *
        org Y:
LastY = *
        if @cvi(LastX)>=@cvi(LastY)
          org L:LastX
        else
          org L:LastY
        endif

;*************************************************************************
;External Y-only data RAM

        if EVM56K
          LOMEM Y:$0200
          HIMEM Y:$1FFF
        else
          LOMEM Y:$2000
          HIMEM Y:$3FFF
        endif

      if EVM56K
        org Y:$200
      else
        org Y:$2000
      endif

        org Y:
AliasFilterI dsm AliasFilterLen
AliasFilterQ dsm AliasFilterLen

        org Y:
RxSymbolShape  dsm SymbolShapeLen
TxSymbolShape  dsm SymbolShapeLen
TxCwSymbolShape dsm SymbolShapeLen

      if SendFixedMessage
        opt nops
        org Y:
FixedMessage
             dc CtrlT
             dc '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~',CR
             dc 'CQ CQ CQ slow BPSK de Doug, SV/N1OWU, Athens - Greece',CR
             dc CR,CtrlZ,CtrlR
             dc '                                                     '
             dc '                                                     '
             dc '                   '
             dc 0
      endif


CwIdString
	; PSK DE SV/N1OWU
	dc CW_SPC,CW_P,CW_S,CW_K,CW_SPC,CW_D,CW_E,CW_SPC
	dc CW_S,CW_V,CW_STK,CW_N,CW_1,CW_O,CW_W,CW_U,CW_SPC
CwIdStringEnd
	dc 0

;*************************************************************************
;constant tables: FIR and window shapes

LowFreq equ AliasLowFreq/SampleFreq
UppFreq equ AliasUppFreq/SampleFreq

w0      equ     0.35875 ;coeff. for Blackman-Harris 4 term
w1      equ     0.48829 ;minimum sidelobe window (copied from Motorola BBS)
w2      equ     0.14128
w3	equ	0.01168

        org Y:AliasFilterI

time = -@cvf(AliasFilterLen/2)+0.5
count   set 0
        dup AliasFilterLen
angle = PI*time/@cvf(AliasFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = (@sin(2.0*PI*time*UppFreq)-@sin(2.0*PI*time*LowFreq))/(PI*time)
time = time+1.0
        dc  Window*Filter
count   set count+1
        endm

        org Y:AliasFilterQ

time = -@cvf(AliasFilterLen/2)+0.5
count   set 0
        dup AliasFilterLen
angle = PI*time/@cvf(AliasFilterLen/2)
Window = w0+w1*@cos(angle)+w2*@cos(2.0*angle)+w3*@cos(3.0*angle)
Filter = (-@cos(2.0*PI*time*UppFreq)+@cos(2.0*PI*time*LowFreq))/(PI*time)
time = time+1.0
        dc  Window*Filter
count   set count+1
        endm

s0      equ     0.35005138      ;Like Blackman-Harris but for minimal
s1      equ     0.50799695      ;crosstalk between symbols.
s2      equ     0.14499812      ;The Blackman-Harris is not that bad and
s3      equ    -0.01294745      ;the transmitted spectrum has no sidelobes.

        org Y:RxSymbolShape

time = -@cvf(SymbolShapeLen/2)+0.5
count   set 0
        dup SymbolShapeLen
angle = PI*time/@cvf(SymbolShapeLen/2)
Filter = s0+s1*@cos(angle)+s2*@cos(2.0*angle)+s3*@cos(3.0*angle)
;Filter = 0.75*@cos(PI*time/@cvf(SymbolShapeLen))+0.25*@cos(3*PI*time/@cvf(SymbolShapeLen))
time = time+1.0
        dc  Filter/(SymbolShapeLen/2)
count   set count+1
        endm

        org Y:TxSymbolShape

time = -@cvf(SymbolShapeLen/2)+0.5
count   set 0
        dup SymbolShapeLen
angle = PI*time/@cvf(SymbolShapeLen/2)
;Filter = 0.5+0.5*@cos(angle)
Filter = s0+s1*@cos(angle)+s2*@cos(2.0*angle)+s3*@cos(3.0*angle)
;Filter = 0.75*@cos(PI*time/@cvf(SymbolShapeLen))+0.25*@cos(3*PI*time/@cvf(SymbolShapeLen))
time = time+1.0
        dc  Filter/2
count   set count+1
        endm

        org Y:TxCwSymbolShape
        opt mex

CwRamp  equ 10	  ;determines rise and fall times of CW waveform
CwMax   equ 0.475 ;should be set to get the same PEP for CW and PSK

; This generates simple linear ramps.  Some sort of sine-like
; curve might be nicer...  Keep in mind that the summed weights of
; overlapping symbols must be constant, so that dashes are smooth...

	dup 64-CwRamp
	dc 0.0
	endm

count   set 1
	dup 2*CwRamp
Filter = count/(2.0*CwRamp+1)
	dc Filter*CwMax
count   set count+1
        endm

	dup 128-(2*CwRamp)
	dc CwMax
	endm

count   set 2*CwRamp
	dup 2*CwRamp
Filter = count/(2.0*CwRamp+1)
	dc Filter*CwMax
count   set count-1
        endm

	dup 64-CwRamp
	dc 0.0
	endm

;*************************************************************************

        end

